/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * ActuatorData.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: ActuatorData.java,v 1.2 2005/01/25 23:57:20 jaquier Exp $
 */

package birg.bluemove.communication.core;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: This class is used to store a position of an
 * actuator.</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 * @author Cyril
 * @version 1.0
 */
public class ActuatorData extends ModuleDataComponent {

    private double m_value = 0.0;

    /**
     * Constructor for an ActuatorData. Contains the value to transmit to the
     * module. There is one ActuatorData for each actuator.
     *
     * @param name String the name of the actuator
     */
    public ActuatorData(String name) {
        super(name);
    }

    /**
     * Another constructor.
     *
     * @param name String the name of the actuator
     * @param value double the value to transmit
     */
    public ActuatorData(String name, double value) {
        this(name);
        m_value = value;
    }

    /**
     * Gets the value of the actuator.
     *
     * @return double
     */
    public double getValue() {
        return m_value;
    }

    /**
     * Sets the value of the actuator.
     *
     * @param value double
     */
    public void setValue(double value) {
        m_value = value;
    }
}
