/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ |/ / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * DataInterface.java
 *
 * Created on November 12, 2004, 1:38 PM
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: DataInterface.java,v 1.3 2005/01/25 23:57:20 jaquier Exp $
 */

package birg.bluemove.communication.core;

import java.util.LinkedList;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: This abstract class must be extended for each communcation
 * device. It defines the methods used to communicate with and send data to a
 * module.</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 * @author Cyril
 * @version 1.0
 */
public abstract class DataInterface {

    protected LinkedList<ModuleData> m_moduleList = null;

    /**
     * Constructor. Creates the initial module list (LinkedList).
     */
    public DataInterface() {
        m_moduleList = new LinkedList<ModuleData>();
    }

    /**
     * Returns the modules list.
     *
     * @return LinkedList
     */
    public LinkedList<ModuleData> getModuleList() {
        return m_moduleList;
    }

    /**
     * Sends data to all the devices present in the ModuleDataManager. Return
     * true in case of success else return false.
     *
     * @param manager ModuleDataManager
     * @return boolean
     */
    public abstract boolean sendData(ModuleDataManager manager);

    /**
     * Starts a serial connection to all the modules in ModuleDataManager. This
     * method must be called before sending any data.
     *
     * @param manager ModuleDataManager
     * @return boolean
     */
    public abstract boolean startConnections(ModuleDataManager manager);

    /**
     * Stops all the connections previously opened by startConnections().
     *
     * @return boolean
     */
    public abstract boolean stopConnections();

    /**
     * Starts an inquiry. This should get all the available modules in the area.
     */
    public abstract void inquiry();

    /**
     * Stops a pending inquiry.
     */
    public abstract void stopInquiry();
}
