/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ |/ / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * ModuleData.java
 *
 * Created on November 15, 2004, 8:42 PM
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: ModuleData.java,v 1.3 2005/01/25 23:57:20 jaquier Exp $
 */

package birg.bluemove.communication.core;

import java.util.LinkedList;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: Contains all the data needed to set the state of a module.
 * Can have<ul>
 * <li>multiple fields (servo1, servo2, led, ...). Should also be used to</li>
 * <li>store sensors values from module (distance, temp, ...).</li></ul></p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 * @author Cyril
 * @version 1.0
 */
public class ModuleData {

    private String m_address = null;
    private LinkedList<ModuleDataComponent> m_components = null;


    /**
     * Constructor. Corresponds to a module. Stores the address of the module
     * and a list of depending components.
     *
     * @param address String
     */
    public ModuleData(String address) {
        m_address = address;
        m_components = new LinkedList<ModuleDataComponent>();
    }

    /**
     * Sets the address of the module.
     *
     * @param address String
     */
    public void setAddress(String address) {
        m_address = address;
    }

    /**
     * Returns the address sets by setAddress() or by the constructor.
     *
     * @return String
     */
    public String getAddress() {
        return m_address;
    }

    /**
     * Adds a component to a ModuleData.
     *
     * @param component ModuleDataComponent
     */
    public void addComponent(ModuleDataComponent component) {
        m_components.add(component);
    }

    /**
     * Removes an existing component from the module components list.
     *
     * @param component ModuleDataComponent
     */
    public void removeComponent(ModuleDataComponent component) {
        m_components.remove(component);
    }

    /**
     * Gets all the components for this module.
     *
     * @return LinkedList
     */
    public LinkedList<ModuleDataComponent> getComponentList() {
        return m_components;
    }

    /**
     * Gets a specified component which belongs to the module. If the param
     * "name" does not correspond to an existing component, returns null.
     *
     * @param name String
     * @return ModuleDataComponent
     */
    public ModuleDataComponent getComponent(String name) {
        for (ModuleDataComponent element : m_components) {
            if (name.equals(element.getName())) {
                return element;
            }
        }
        return null;
    }
}
