/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * ModuleDataManager.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: ModuleDataManager.java,v 1.3 2005/01/25 23:57:20 jaquier Exp $
 */

package birg.bluemove.communication.core;

import java.util.LinkedList;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: This class corresponds to the ModuleManager. It is a
 * singleton which contains all the modules which will receive data.</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 * @author Cyril
 * @version 1.0
 */
public class ModuleDataManager {
    static private ModuleDataManager m_instance = null;

    private LinkedList<ModuleData> m_modules = null;

    /**
     * Constructor. Creates the initial ModuleData list. It is private because
     * it should not be access from outside this class. Eh guy, we said it was a
     * singleton.
     */
    private ModuleDataManager() {
        m_modules = new LinkedList<ModuleData>();
    }

    /**
     * Returns the ModuleDataManager singleton. This method must be used. Do not
     * use: <pre>ModuleDataManager xyz = new ModuleDataManager();</pre>.
     *
     * @return ModuleDataManager
     */
    static public ModuleDataManager getInstance() {
        if (m_instance == null) {
            m_instance = new ModuleDataManager();
        }
        return m_instance;
    }

    /**
     * Adds a module to the manager.
     *
     * @param module ModuleData
     */
    public void addModule(ModuleData module) {
        m_modules.add(module);
    }

    /**
     * Removes a given module from the manager.
     *
     * @param module ModuleData
     */
    public void removeModule(ModuleData module) {
        m_modules.remove(module);
    }

    /**
     * Gets the module list.
     *
     * @return LinkedList
     */
    public LinkedList<ModuleData> getModuleList() {
        return m_modules;
    }

    /**
     * Clears the modules list. Used instead of deleting/creating a
     * ModuleDataManager.
     */
    public void clear() {
        m_modules.clear();
    }

}
