/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * ModulesList.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: ModulesList.java,v 1.2 2005/01/25 23:57:20 jaquier Exp $
 */

package birg.bluemove.communication.gui;

import java.awt.Component;
import java.util.LinkedList;

import javax.bluetooth.BluetoothStateException;

import birg.bluemove.communication.bluetooth.Bluetooth;
import birg.bluemove.communication.core.DataInterface;
import birg.bluemove.communication.core.ModuleData;
import birg.bluemove.utils.gui.MessageBox;
import org.apache.log4j.Logger;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: This class is an "application layer" between core and GUI. It
 * creates a new ModulesListGUI which allows the user to inquiry and select a
 * module.</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 * @author Cyril
 * @version 1.0
 */
public class ModulesList {

    static Logger logger = Logger.getLogger(ModulesList.class);

    ModulesListGUI m_gui = null;
    DataInterface m_bT = null;

    /**
     * Constructor. Creates a new ModulesListGUI and get the Bluetooth instance.
     * If Bluetooth system is not available, displays an error message box.
     *
     * @param component Component
     */
    public ModulesList(Component component) {
        try {
            m_bT = Bluetooth.getInstance();
            m_gui = new ModulesListGUI(this);
            m_gui.setLocationRelativeTo(component);
            m_gui.setVisible(true);
        } catch (BluetoothStateException e) {
            logger.error(e.getMessage());
            String msg = "Unable to find a Bluetooth device. Please check " +
                         "that your Bluetooth device is correctly initialized.";
            MessageBox mBox = new MessageBox(null,
                                             MessageBox.MessageBoxType.ERROR,
                                             msg);
            mBox.showDialog();
        }
    }

    /**
     * Returns the modules list generated by a Bluetooth inquiry.
     *
     * @return LinkedList
     */
    public LinkedList<ModuleData> getModulesList() {
        return m_bT.getModuleList();
    }

    /**
     * Starts the inquiry.
     */
    public void inquiry() {
        m_bT.inquiry();
    }

    /**
     * Stops the inquiry.
     */
    public void stopInquiry() {
        m_bT.stopInquiry();
    }

    /**
     * Gets the value selected by the user.
     *
     * @return String
     */
    public String getValue() {
        if (m_gui != null) {
            return m_gui.getValue();
        } else {
            return null;
        }
    }
}
