/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ |/ / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * ModuleList.java
 *
 * Created on 10. decembre 2004, 22:48
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: ModulesListGUI.java,v 1.2 2005/01/25 23:57:20 jaquier Exp $
 */

package birg.bluemove.communication.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import birg.bluemove.communication.core.ModuleData;
import birg.bluemove.utils.gui.SwingWorker;
import org.apache.log4j.Logger;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: This class is the GUI which allows the user to inquiry and
 * select an address from the discovered modules list.</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 * @author Cyril
 * @version 1.0
 */
public class ModulesListGUI extends JDialog {

    static Logger logger = Logger.getLogger(ModulesListGUI.class);

    private String m_address = null;
    private ModulesList m_modulesList = null;
    private Boolean m_isInquiring = false;
    private SwingWorker m_worker = null;

    /**
     * Constructor. Creates the graphical components and updates the address
     * list.
     *
     * @param modulesList ModulesList
     */
    public ModulesListGUI(ModulesList modulesList) {
        m_modulesList = modulesList;
        try {
            jbInit();
        } catch (Exception ex) {
            logger.error(ex.getMessage());
        }
        updateList();
    }

    /**
     * Checks if the system is currently inquiring.
     *
     * @return boolean
     */
    private boolean getInquiry() {
        synchronized (m_isInquiring) {
            return m_isInquiring;
        }
    }

    /**
     * Sets the inquiry state.
     *
     * @param value boolean
     */
    private void setInquiry(boolean value) {
        synchronized (m_isInquiring) {
            m_isInquiring = value;
        }

    }

    /**
     * Sets the graphical components.
     *
     * @throws Exception
     */
    private void jbInit() throws Exception {
        this.getContentPane().setLayout(borderLayout1);
        jButton1.setText("Add");
        jButton1.setEnabled(false);
        jButton1.addActionListener(new ModuleList_jButton1_actionAdapter(this));
        jScrollPane1.setDoubleBuffered(true);
        this.setModal(true);
        this.setAlwaysOnTop(true);
        this.setTitle("Module Selection");
        jButton3.setText("Cancel");
        jButton3.addActionListener(new ModuleList_jButton3_actionAdapter(this));
        jButton2.addActionListener(new ModuleList_jButton2_actionAdapter(this));
        jList1.addListSelectionListener(new
                                        ModulesListGUI_jList1_listSelectionAdapter(this));
        this.getContentPane().add(jPanel1, java.awt.BorderLayout.SOUTH);
        jButton2.setText("Refresh");
        jList1.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        jList1.setModel(listModel);
        jPanel1.add(jButton2);
        jPanel1.add(jButton1);
        jPanel1.add(jButton3);
        this.getContentPane().add(jScrollPane1, java.awt.BorderLayout.CENTER);
        jScrollPane1.getViewport().add(jList1);
        pack();
    }

    BorderLayout borderLayout1 = new BorderLayout();
    JPanel jPanel1 = new JPanel();
    JButton jButton1 = new JButton();
    JButton jButton2 = new JButton();
    JList jList1 = new JList();
    DefaultListModel listModel = new DefaultListModel();
    JScrollPane jScrollPane1 = new JScrollPane();
    JButton jButton3 = new JButton();

    /**
     * Gets the selected address.
     *
     * @return String
     */
    public String getValue() {
        return m_address;
    }

    /**
     * Update the list with addresses already present in the cache.
     */
    private void updateList() {
        listModel.clear();
        if (m_modulesList.getModulesList() != null) {
            for (ModuleData element : m_modulesList.getModulesList()) {
                listModel.addElement(element.getAddress());
            }
        }
    }

    /**
     * Action performed on the Cancel/Close button.
     *
     * @param e ActionEvent
     */
    public void jButton3_actionPerformed(ActionEvent e) {
        if (m_isInquiring) {
            m_modulesList.stopInquiry();
            m_worker.interrupt();
            m_worker.finished();
        } else {
            dispose();
        }
    }

    /**
     * Action performed on the Add button.
     *
     * @param e ActionEvent
     */
    public void jButton1_actionPerformed(ActionEvent e) {
        // Add
        m_address = jList1.getSelectedValue().toString();
        setVisible(false);
    }

    /**
     * Action performed on the Refresh button. Uses a thread in order to allow
     * the user to cancel the inquiry.
     *
     * @param e ActionEvent
     * @todo Cleanup this messy method.
     */
    public void jButton2_actionPerformed(ActionEvent e) {
        // Refresh
        m_worker = new SwingWorker() {
            public Object construct() {
                jButton1.setEnabled(false);
                jButton2.setEnabled(false);

                SwingWorker spinner = new SwingWorker() {
                    public Object construct() {
                        String first = "Inquiring ";
                        String anim[] = {"\\", "|", "/", "-"};
                        int i = 0;
                        while (getInquiry()) {
                            listModel.clear();
                            listModel.addElement(first + anim[i]);
                            i = (i + 1) % anim.length;
                            try {
                                Thread.sleep(100);
                            } catch (Exception e) {
                            }
                        }
                        return null;
                    }
                };
                setInquiry(true);
                spinner.start();
                m_modulesList.inquiry();
                return null;
            }

            public void finished() {
                setInquiry(false);
                updateList();
                jButton2.setEnabled(true);
            }
        };
        m_worker.start();
    }

    /**
     * Value changed in the address list.
     *
     * @param e ListSelectionEvent
     */
    public void jList1_valueChanged(ListSelectionEvent e) {
        if (!getInquiry()) {
            jButton1.setEnabled(true);
        }
    }
}


class ModulesListGUI_jList1_listSelectionAdapter implements
        ListSelectionListener {
    private ModulesListGUI adaptee;
    ModulesListGUI_jList1_listSelectionAdapter(ModulesListGUI adaptee) {
        this.adaptee = adaptee;
    }

    public void valueChanged(ListSelectionEvent e) {
        adaptee.jList1_valueChanged(e);
    }
}


class ModuleList_jButton2_actionAdapter implements ActionListener {
    private ModulesListGUI adaptee;
    ModuleList_jButton2_actionAdapter(ModulesListGUI adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.jButton2_actionPerformed(e);
    }
}


class ModuleList_jButton3_actionAdapter implements ActionListener {
    private ModulesListGUI adaptee;
    ModuleList_jButton3_actionAdapter(ModulesListGUI adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.jButton3_actionPerformed(e);
    }
}


class ModuleList_jButton1_actionAdapter implements ActionListener {
    private ModulesListGUI adaptee;
    ModuleList_jButton1_actionAdapter(ModulesListGUI adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.jButton1_actionPerformed(e);
    }
}
