/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * Config.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: Config.java,v 1.4 2005/01/25 23:55:12 jaquier Exp $
 */

package birg.bluemove.config.core;

import java.io.*;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: This interface defines methods which must be implemented by
 * each archive system (Zip, tar, RAR, ...).</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 * @author Cyril
 * @version 1.0
 */
public interface Config {
    /**
     * Reads a file from an archive.
     *
     * @param fName String
     * @return InputStream
     */
    public InputStream readFromArchive(String fName);

    /**
     * Checks if the given file exists in the loaded archive.
     *
     * @param fName String
     * @return boolean
     */
    public boolean fileExists(String fName);

    /**
     * Write a stream to the archive. Used for creating a new file or modifing
     * an existing one.
     *
     * @param oS OutputStream
     * @param fName String
     * @throws IOException
     */
    public void writeToArchive(OutputStream oS, String fName) throws
            IOException;

    /**
     * Loads an archive in memory. This method must be called before any access
     * to the archive.
     *
     * @param aName String
     * @return boolean
     */
    public boolean loadArchive(String aName);

    /**
     * Creates a new archive.
     *
     * @param aName String
     */
    public void createArchive(String aName);
}
