/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * ModulesImage.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: ModulesImage.java,v 1.3 2005/01/25 23:55:12 jaquier Exp $
 */

package birg.bluemove.config.core;

import java.awt.image.BufferedImage;
import java.io.IOException;

import birg.bluemove.module.core.ModulesManager;
import birg.bluemove.utils.gui.GUITools;
import org.apache.log4j.Logger;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: This class manages the persistance of the modules image. This
 * image is displayed in the project setting and should allow the user to build
 * the modular robot easily.</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 * @author Cyril
 * @version 1.0
 */
public class ModulesImage implements Persistance {

    static Logger logger = Logger.getLogger(ModulesImage.class);

    Config m_config = null;
    ModulesManager m_mManager = ModulesManager.getInstance();

    /**
     * Constructor. Gets the reference to the persistance support.
     *
     * @param config Config
     */
    public ModulesImage(Config config) {
        m_config = config;
    }

    /**
     * Gets the image from the archive.
     */
    public void get() {
        BufferedImage image = null;
        if (m_config.fileExists("images/model.png")) {
            try {
                image = GUITools.readImage(m_config.readFromArchive(
                        "images/model.png"));
                m_mManager.setBuildImage(image);
            } catch (IOException e) {
                logger.error(e.getMessage());
            }
        } else {
            logger.warn("Image project does not exist");
        }
        m_mManager.setBuildImage(image);
    }

    /**
     * Copies the image to the archive.
     *
     * @todo Should be implemented.
     */
    public void set() {
    }

}
