/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * OpenProject.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: OpenProject.java,v 1.9 2005/01/25 23:55:12 jaquier Exp $
 */

package birg.bluemove.config.core;

import java.io.File;

import birg.bluemove.config.gui.OpenProjectDialog;
import birg.bluemove.utils.gui.MessageBox;
import org.apache.log4j.Logger;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: This class is used when opening a project. It should call the
 * get() method of all the peristance classes needed in order to restore a
 * project.</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 * @author Cyril
 * @version 1.0
 */
public class OpenProject {

    static Logger logger = Logger.getLogger(OpenProject.class);

    protected Config m_config = ZipConfig.getInstance();
    protected OpenProjectDialog m_projectDialog = new OpenProjectDialog(this);

    /**
     * Constructor.
     */
    public OpenProject() {
    }

    /**
     * Sets the current directory of the browser dialog in order to get into the
     * last directory.
     *
     * @param current String
     */
    public void setCurrentDir(String current) {
        logger.debug("Set current directory: " + current);
        m_projectDialog.setCurrentDirectory(new File(current));
    }

    /**
     * Opens a project.
     *
     * @param file File
     * @return boolean
     */
    public boolean openProjectFile(File file) {
        if (!m_config.loadArchive(file.getAbsolutePath())) {
            MessageBox mBox = new MessageBox(null,
                                             MessageBox.MessageBoxType.ERROR,
                                             "Unable to open " +
                                             file.getAbsoluteFile());
            mBox.showDialog();
            return false;
        }

        XMLModuleManager mManager = new XMLModuleManager(m_config);
        mManager.get();

        XMLTimelinesManager tlManager = new XMLTimelinesManager(m_config);
        tlManager.get();

        ModulesImage mImage = new ModulesImage(m_config);
        mImage.get();

        return true;
    }

    /**
     * Shows the save browser dialog.
     *
     * @return File
     */
    public File showDialog() {
        return m_projectDialog.showAndGet(null);
    }
}
