/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * SaveProject.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: SaveProject.java,v 1.8 2005/01/25 23:55:12 jaquier Exp $
 */

package birg.bluemove.config.core;

import java.io.File;

import birg.bluemove.config.gui.SaveProjectDialog;
import org.apache.log4j.Logger;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: This class is used when saving a project. It should call the
 * set() method of all the peristance classes needed in order to save a project.
 * </p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 * @author Cyril
 * @version 1.0
 */
public class SaveProject {

    static Logger logger = Logger.getLogger(SaveProject.class);

    protected Config m_config = ZipConfig.getInstance();
    protected SaveProjectDialog m_projectDialog = new SaveProjectDialog(this);

    /**
     * Constructor.
     */
    public SaveProject() {
    }

    /**
     * Sets the current directory of the browser dialog in order to get into the
     * last directory.
     *
     * @param current String
     */
    public void setCurrentDir(String current) {
        logger.debug("Set current directory: " + current);
        m_projectDialog.setCurrentDirectory(new File(current));
    }

    /**
     * Saves a project. If the param <pre>saveAs</pre> is true, a new archive
     * will be created. Else the current one is overwritten with the new
     * settings.
     *
     * @param file File
     * @param saveAs boolean
     * @return boolean
     */
    public boolean saveProjectFile(File file, boolean saveAs) {
        if (saveAs) {
            m_config.createArchive(file.getAbsolutePath());
        }
        if (!m_config.loadArchive(file.getAbsolutePath())) {
            return false;
        }
        XMLModuleManager mManager = new XMLModuleManager(m_config);
        XMLTimelinesManager tlManager = new XMLTimelinesManager(m_config);
        mManager.set();
        tlManager.set();

        return true;
    }

    /**
     * Shows the save browser dialog.
     *
     * @return File
     */
    public File showDialog() {
        return m_projectDialog.showAndGet(null);
    }
}
