/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * PrefGeneral.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: PrefsGeneral.java,v 1.4 2005/01/27 16:37:09 jaquier Exp $
 */

package birg.bluemove.config.gui;

import java.awt.*;
import java.awt.event.*;
import java.util.prefs.Preferences;

import javax.swing.*;

import birg.bluemove.main.core.Bluemove;
import birg.bluemove.resources.resources;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: This class displays a preferences panel for the general
 * Bluemove parameters.</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 * @author Cyril
 * @version 1.0
 */
public class PrefsGeneral extends JPanel implements Setting {

    Preferences m_prefs = null;
    PrefsGUI m_prefsGui = null;

    /**
     * Constructor.
     *
     * @param prefsGui PrefsGUI
     */
    public PrefsGeneral(PrefsGUI prefsGui) {
        m_prefs = Preferences.userNodeForPackage(Bluemove.class);
        m_prefsGui = prefsGui;

        try {
            jbInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * Loads the settings into the graphical components.
     *
     * @param obj Object
     */
    public void loadSettings(Object obj) {
        // Text antialiasing
        if (m_prefs.getBoolean("aatext", false)) {
            jCheckBox1.setSelected(true);
        } else {
            jCheckBox1.setSelected(false);
        }

        // Internet browser
        jTextField1.setText(m_prefs.get("browser", "firefox"));

        // Disable button as nothing could have changed
        jButton1.setEnabled(false);
    }

    /**
     * Saves the user settings.
     */
    public void saveSettings() {
        // Text antialiasing
        if (jCheckBox1.isSelected()) {
            m_prefs.putBoolean("aatext", true);
        } else {
            m_prefs.putBoolean("aatext", false);
        }

        // Internet browser
        m_prefs.put("browser", jTextField1.getText());

        // Disable button as nothing could have changed
        jButton1.setEnabled(false);
    }

    /**
     * Initializes the graphical components.
     *
     * @throws Exception
     */
    private void jbInit() throws Exception {
        this.setLayout(borderLayout1);
        jButton1.setText("Apply");
        jButton1.setMaximumSize(new Dimension(95, 25));
        jButton1.setMinimumSize(new Dimension(95, 25));
        jButton1.setPreferredSize(new Dimension(95, 25));
        jButton1.addActionListener(new PrefsGeneral_jButton1_actionAdapter(this));
        jButton1.setIcon(new ImageIcon(resources.class.getResource(
                "icon_accept.png")));
        jPanel2.setLayout(borderLayout2);
        jLabel1.setText("General settings");
        jPanel3.setLayout(gridLayout1);
        jCheckBox1.setText("Activate text antialiasing");
        jCheckBox1.addActionListener(new PrefsGeneral_jCheckBox1_actionAdapter(this));
        jLabel2.setText("Internet browser");
        jTextField1.addKeyListener(new PrefsGeneral_jTextField1_keyAdapter(this));
        jButton2.setMaximumSize(new Dimension(95, 25));
        jButton2.setMinimumSize(new Dimension(95, 25));
        jButton2.setPreferredSize(new Dimension(95, 25));
        jButton2.setMnemonic('0');
        jButton2.setText("Cancel");
        jButton2.addActionListener(new PrefsGeneral_jButton2_actionAdapter(this));
        jButton2.setIcon(new ImageIcon(resources.class.getResource(
                "icon_cancel.png")));
        jPanel5.setLayout(gridLayout2);
        this.add(jPanel1, java.awt.BorderLayout.SOUTH);
        jPanel1.add(jButton1);
        jPanel1.add(jButton2);
        this.add(jPanel2, java.awt.BorderLayout.NORTH);
        jPanel2.add(jPanel3, java.awt.BorderLayout.CENTER);
        jPanel3.add(jCheckBox1);
        jPanel4.add(jLabel1);
        jPanel2.add(jPanel4, java.awt.BorderLayout.NORTH);
        jPanel2.add(jPanel5, java.awt.BorderLayout.SOUTH);
        jPanel5.add(jLabel2, null);
        jPanel5.add(jTextField1, null);
    }

    BorderLayout borderLayout1 = new BorderLayout();
    JPanel jPanel1 = new JPanel();
    JPanel jPanel2 = new JPanel();
    JButton jButton1 = new JButton();
    JPanel jPanel3 = new JPanel();
    BorderLayout borderLayout2 = new BorderLayout();
    JPanel jPanel4 = new JPanel();
    JLabel jLabel1 = new JLabel();
    GridLayout gridLayout1 = new GridLayout();
    JCheckBox jCheckBox1 = new JCheckBox();
    JButton jButton2 = new JButton();
    JPanel jPanel5 = new JPanel();
    JLabel jLabel2 = new JLabel();
    JTextField jTextField1 = new JTextField();
    GridLayout gridLayout2 = new GridLayout();

    /**
     * Action performed on the <pre>close</pre> button.
     *
     * @param e ActionEvent
     */
    public void jButton1_actionPerformed(ActionEvent e) {
        saveSettings();
        jButton2.setText("Close");
        jButton2.setIcon(new ImageIcon(resources.class.getResource(
                "icon_exit.png")));
    }

    /**
     * The check box for antialiasing option changed.
     *
     * @param e ActionEvent
     */
    public void jCheckBox1_actionPerformed(ActionEvent e) {
        jButton1.setEnabled(true);
    }

    /**
     * Key typed in the Internet browser text field.
     *
     * @param e KeyEvent
     */
    public void jTextField1_keyTyped(KeyEvent e) {
        jButton1.setEnabled(true);
    }

    /**
     * Action performed on the <pre>close</pre> button.
     *
     * @param e ActionEvent
     */
    public void jButton2_actionPerformed(ActionEvent e) {
        m_prefsGui.dispose();
    }
}


class PrefsGeneral_jButton2_actionAdapter implements ActionListener {
    private PrefsGeneral adaptee;
    PrefsGeneral_jButton2_actionAdapter(PrefsGeneral adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.jButton2_actionPerformed(e);
    }
}


class PrefsGeneral_jCheckBox1_actionAdapter implements ActionListener {
    private PrefsGeneral adaptee;
    PrefsGeneral_jCheckBox1_actionAdapter(PrefsGeneral adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.jCheckBox1_actionPerformed(e);
    }
}


class PrefsGeneral_jButton1_actionAdapter implements ActionListener {
    private PrefsGeneral adaptee;
    PrefsGeneral_jButton1_actionAdapter(PrefsGeneral adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.jButton1_actionPerformed(e);
    }
}


class PrefsGeneral_jTextField1_keyAdapter extends KeyAdapter {
    private PrefsGeneral adaptee;
    PrefsGeneral_jTextField1_keyAdapter(PrefsGeneral adaptee) {
        this.adaptee = adaptee;
    }

    public void keyTyped(KeyEvent e) {
        adaptee.jTextField1_keyTyped(e);
    }
}
