/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * PrefGeneral.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: PrefsPlayer.java,v 1.2 2005/01/27 22:03:13 jaquier Exp $
 */

package birg.bluemove.config.gui;

import java.awt.*;
import java.awt.event.*;
import java.util.prefs.Preferences;

import javax.swing.*;

import birg.bluemove.main.core.Bluemove;
import birg.bluemove.resources.resources;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: This class displays a preferences panel for the player. These
 * data should not be saved here. They must be saved into the project
 * archive.</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 * @author Cyril
 * @version 1.0
 *
 * @todo Removes this class and saves the settings in the project archive.
 */
public class PrefsPlayer extends JPanel implements Setting {

    Preferences m_prefs = null;
    PrefsGUI m_prefsGui = null;

    /**
     * Constructor.
     *
     * @param prefsGui PrefsGUI
     */
    public PrefsPlayer(PrefsGUI prefsGui) {
        m_prefs = Preferences.userNodeForPackage(Bluemove.class);
        m_prefsGui = prefsGui;

        try {
            jbInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * Loads the settings.
     *
     * @param obj Object
     */
    public void loadSettings(Object obj) {
        // Increment
        jTextField1.setText(Double.toString(m_prefs.getDouble("playerIncrement",
                0.2)));

        // Sleep
        jTextField2.setText(Long.toString(m_prefs.getLong("playerSleep",
                500)));

        // Disable button as nothing could have changed
        jButton1.setEnabled(false);
    }

    /**
     * Saves the settings.
     */
    public void saveSettings() {
        // Increment
        m_prefs.putDouble("playerIncrement",
                          Double.parseDouble(jTextField1.getText()));

        // Sleep
        m_prefs.putLong("playerSleep", Long.parseLong(jTextField2.getText()));

        // Disable button as nothing could have changed
        jButton1.setEnabled(false);
    }

    /**
     * Initializes the graphical components.
     *
     * @throws Exception
     */
    private void jbInit() throws Exception {
        this.setLayout(borderLayout1);
        jButton1.setText("Apply");
        jButton1.setMaximumSize(new Dimension(95, 25));
        jButton1.setMinimumSize(new Dimension(95, 25));
        jButton1.setPreferredSize(new Dimension(95, 25));
        jButton1.addActionListener(new PrefsPlayer_jButton1_actionAdapter(this));
        jButton1.setIcon(new ImageIcon(resources.class.getResource(
                "icon_accept.png")));
        jPanel2.setLayout(borderLayout2);
        jLabel1.setText("Player settings");
        jLabel2.setText("Increment");
        jButton2.setMaximumSize(new Dimension(95, 25));
        jButton2.setMinimumSize(new Dimension(95, 25));
        jButton2.setPreferredSize(new Dimension(95, 25));
        jButton2.setMnemonic('0');
        jButton2.setText("Cancel");
        jButton2.addActionListener(new PrefsPlayer_jButton2_actionAdapter(this));
        jButton2.setIcon(new ImageIcon(resources.class.getResource(
                "icon_cancel.png")));
        jPanel5.setLayout(gridLayout2);
        jLabel3.setText("Sleep time");
        jPanel6.setLayout(gridLayout3);
        jTextField1.addKeyListener(new PrefsPlayer_jTextField1_keyAdapter(this));
        jTextField2.addKeyListener(new PrefsPlayer_jTextField2_keyAdapter(this));
        this.add(jPanel1, java.awt.BorderLayout.SOUTH);
        jPanel1.add(jButton1);
        jPanel1.add(jButton2);
        this.add(jPanel2, java.awt.BorderLayout.NORTH);
        jPanel4.add(jLabel1);
        jPanel5.add(jLabel2, null);
        jPanel5.add(jTextField1, null);
        jPanel6.add(jLabel3);
        jPanel6.add(jTextField2);

        jPanel2.add(jPanel4, java.awt.BorderLayout.NORTH);
        jPanel2.add(jPanel5, java.awt.BorderLayout.CENTER);
        jPanel2.add(jPanel6, java.awt.BorderLayout.SOUTH);
    }

    BorderLayout borderLayout1 = new BorderLayout();
    JPanel jPanel1 = new JPanel();
    JPanel jPanel2 = new JPanel();
    JButton jButton1 = new JButton();
    BorderLayout borderLayout2 = new BorderLayout();
    JPanel jPanel4 = new JPanel();
    JLabel jLabel1 = new JLabel();
    JButton jButton2 = new JButton();
    JPanel jPanel5 = new JPanel();
    JLabel jLabel2 = new JLabel();
    JTextField jTextField1 = new JTextField();
    GridLayout gridLayout2 = new GridLayout();
    JPanel jPanel6 = new JPanel();
    JLabel jLabel3 = new JLabel();
    JTextField jTextField2 = new JTextField();
    GridLayout gridLayout3 = new GridLayout();


    /**
     * Action performed on the <pre>apply</pre> button.
     *
     * @param e ActionEvent
     */
    public void jButton1_actionPerformed(ActionEvent e) {
        saveSettings();
        jButton2.setText("Close");
        jButton2.setIcon(new ImageIcon(resources.class.getResource(
                "icon_exit.png")));
    }

    /**
     * Key typed on setting text field.
     *
     * @param e KeyEvent
     */
    public void jTextField1_keyTyped(KeyEvent e) {
        jButton1.setEnabled(true);
    }

    /**
     * Key typed on setting text field.
     *
     * @param e KeyEvent
     */
    public void jTextField2_keyTyped(KeyEvent e) {
        jButton1.setEnabled(true);
    }

    /**
     * Action performed on <pre>close</pre> button.
     *
     * @param e ActionEvent
     */
    public void jButton2_actionPerformed(ActionEvent e) {
        m_prefsGui.dispose();
    }
}


class PrefsPlayer_jButton2_actionAdapter implements ActionListener {
    private PrefsPlayer adaptee;
    PrefsPlayer_jButton2_actionAdapter(PrefsPlayer adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.jButton2_actionPerformed(e);
    }
}


class PrefsPlayer_jTextField1_keyAdapter extends KeyAdapter {
    private PrefsPlayer adaptee;
    PrefsPlayer_jTextField1_keyAdapter(PrefsPlayer adaptee) {
        this.adaptee = adaptee;
    }

    public void keyTyped(KeyEvent e) {
        adaptee.jTextField1_keyTyped(e);
    }
}


class PrefsPlayer_jTextField2_keyAdapter extends KeyAdapter {
    private PrefsPlayer adaptee;
    PrefsPlayer_jTextField2_keyAdapter(PrefsPlayer adaptee) {
        this.adaptee = adaptee;
    }

    public void keyTyped(KeyEvent e) {
        adaptee.jTextField2_keyTyped(e);
    }
}


class PrefsPlayer_jButton1_actionAdapter implements ActionListener {
    private PrefsPlayer adaptee;
    PrefsPlayer_jButton1_actionAdapter(PrefsPlayer adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.jButton1_actionPerformed(e);
    }
}
