/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * Console.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: Console.java,v 1.6 2005/01/27 22:04:07 jaquier Exp $
 */

package birg.bluemove.logging.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.ByteArrayOutputStream;

import javax.swing.*;

import birg.bluemove.resources.resources;
import birg.bluemove.utils.gui.SwingWorker;
import org.apache.log4j.*;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: This class displays log messages in a nice console. It is
 * highly inspired from the Azureus console. The log messages come from a log4j
 * writer. There is still some strange bugs that probably come from the running
 * thread.</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 * @author Cyril
 * @version 1.0
 */
public class Console extends JPanel {

    static Logger logger = Logger.getLogger(Console.class);

    private ByteArrayOutputStream m_oS = null;
    private SwingWorker m_worker = null;
    private DefaultListModel m_listModel = null;
    private ConsoleCellRenderer m_renderer = null;
    private int m_sleep = 1000;

    /**
     * Constructor. Starts the worker thread which update the messages list.
     */
    public Console() {
        try {
            jbInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }

        m_renderer = new ConsoleCellRenderer();
        m_listModel = new DefaultListModel();
        jList1.setModel(m_listModel);
        jList1.setCellRenderer(m_renderer);

        initialize();
        startWorker();
    }

    /**
     * Initializes the writer appender. We use a specific pattern layout for the
     * display message.
     */
    private void initialize() {
        WriterAppender appender = null;
        PatternLayout pLayout = null;

        m_oS = new ByteArrayOutputStream();
        pLayout = new PatternLayout("%-5p [%t]: %m%n");
        try {
            appender = new WriterAppender(pLayout, m_oS);
            appender.setImmediateFlush(true);
            Logger.getRootLogger().addAppender(appender);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Starts a worker thread. We use the small <pre>SwingWorker</pre> class
     * writing by Sun. The conversion between the outputstream and the string
     * array is quite tricky.
     *
     * @todo This method is really bad coded and buggy. Improve it.
     */
    private void startWorker() {
        m_worker = new SwingWorker() {
            public Object construct() {
                while (true) {
                    if (m_oS.size() > 0) {
                        String sArray[] = m_oS.toString().split("\n");
                        for (String element : sArray) {
                            m_listModel.add(0, element);
//                            m_listModel.addElement(element);
//                            jList1.ensureIndexIsVisible(m_listModel.size() - 1);
                        }
//                        jList1.invalidate();
                        m_oS.reset();
                    }
                    try {
                        // Seems to be a problem with this sleep. Maybe we
                        // should start another thread which update JList1 !?
                        Thread.sleep(m_sleep);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }
        };
        m_worker.start();
    }

    /**
     * Initializes the graphical components.
     *
     * @throws Exception
     */
    private void jbInit() throws Exception {
        this.setLayout(borderLayout1);
        jButton1.setText("Clear");
        jButton1.addActionListener(new Console_jButton1_actionAdapter(this));
        jPanel1.setLayout(borderLayout2);
        jLabel1.setText("Refresh");
        jLabel2.setText("ms");
        jComboBox1.addItemListener(new Console_jComboBox1_itemAdapter(this));
        jComboBox1.setSelectedIndex(3);
        jLabel3.setText("Level");
        jComboBox2.addItemListener(new Console_jComboBox2_itemAdapter(this));
        jComboBox2.setSelectedIndex(2);
        this.add(jScrollPane1, java.awt.BorderLayout.CENTER);
        this.add(jPanel1, java.awt.BorderLayout.SOUTH);
        jScrollPane1.getViewport().add(jList1);
        jPanel2.add(jButton1);
        jPanel1.add(jPanel2, java.awt.BorderLayout.EAST);
        jPanel3.add(jLabel3);
        jPanel3.add(jComboBox2);
        jPanel3.add(jLabel1);
        jPanel3.add(jComboBox1);
        jPanel3.add(jLabel2);
        jPanel1.add(jPanel3, java.awt.BorderLayout.WEST);
    }

    BorderLayout borderLayout1 = new BorderLayout();
    JScrollPane jScrollPane1 = new JScrollPane();
    JPanel jPanel1 = new JPanel();
    JList jList1 = new JList();
    JButton jButton1 = new JButton();
    BorderLayout borderLayout2 = new BorderLayout();
    JPanel jPanel2 = new JPanel();
    JPanel jPanel3 = new JPanel();
    JLabel jLabel1 = new JLabel();
    Integer refreshRate[] = {10, 100, 500, 1000, 2000, 5000};
    JComboBox jComboBox1 = new JComboBox(refreshRate);
    JLabel jLabel2 = new JLabel();
    JLabel jLabel3 = new JLabel();
    Level level[] = {Level.ALL, Level.DEBUG, Level.INFO, Level.WARN,
                    Level.ERROR, Level.FATAL, Level.OFF};
    JComboBox jComboBox2 = new JComboBox(level);

    /**
     * Action performed on "Clear" button.
     *
     * @param e ActionEvent
     */
    public void jButton1_actionPerformed(ActionEvent e) {
        m_listModel.clear();
    }

    /**
     * The state of the refresh rate combobox changed.
     *
     * @param e ItemEvent
     */
    public void jComboBox1_itemStateChanged(ItemEvent e) {
        m_sleep = (Integer) jComboBox1.getSelectedItem();
        logger.info("Set console refresh time to " + m_sleep + " ms");
    }

    /**
     * The state of the log level combobox changed.
     *
     * @param e ItemEvent
     */
    public void jComboBox2_itemStateChanged(ItemEvent e) {
        Logger.getRootLogger().setLevel((Level) jComboBox2.getSelectedItem());
        logger.info("Set debug level to " + jComboBox2.getSelectedItem());
    }
}


class Console_jButton1_actionAdapter implements ActionListener {
    private Console adaptee;
    Console_jButton1_actionAdapter(Console adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.jButton1_actionPerformed(e);
    }
}


class Console_jComboBox1_itemAdapter implements ItemListener {
    private Console adaptee;
    Console_jComboBox1_itemAdapter(Console adaptee) {
        this.adaptee = adaptee;
    }

    public void itemStateChanged(ItemEvent e) {
        adaptee.jComboBox1_itemStateChanged(e);
    }
}


class Console_jComboBox2_itemAdapter implements ItemListener {
    private Console adaptee;
    Console_jComboBox2_itemAdapter(Console adaptee) {
        this.adaptee = adaptee;
    }

    public void itemStateChanged(ItemEvent e) {
        adaptee.jComboBox2_itemStateChanged(e);
    }
}


/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: This class implements the <pre>ListCellRenderer</pre> in
 * order to get more good-looking list rendering.</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 * @author Cyril
 * @version 1.0
 */
class ConsoleCellRenderer extends JLabel implements ListCellRenderer {
    /**
     * Constructor. Sets the list elements to opaque.
     */
    public ConsoleCellRenderer() {
        setOpaque(true);
    }

    /**
     * Draws the list in an Azureus way. Different colors and icons for each log
     * level.
     *
     * @param list JList
     * @param value Object
     * @param index int
     * @param isSelected boolean
     * @param cellHasFocus boolean
     * @return Component
     */
    public Component getListCellRendererComponent(JList list, Object value,
                                                  int index, boolean isSelected,
                                                  boolean cellHasFocus) {
        String sValue = (String) value;

        setFont(new Font("monospaced", Font.PLAIN, 10));
        setText(sValue);
        if (sValue.startsWith("DEBUG")) {
            setIcon(new ImageIcon(resources.class.getResource(
                    "console_question.png")));
            setBackground(new Color(0.9f, 0.9f, 1.0f));
        } else if (sValue.startsWith("INFO")) {
            setIcon(new ImageIcon(resources.class.getResource(
                    "console_info.png")));
            setBackground(new Color(0.75f, 1.0f, 0.7f));
        } else if (sValue.startsWith("WARN")) {
            setIcon(new ImageIcon(resources.class.getResource(
                    "console_warning.png")));
            setBackground(Color.YELLOW);
        } else if (sValue.startsWith("ERROR")) {
            setIcon(new ImageIcon(resources.class.getResource(
                    "console_error.png")));
            setBackground(Color.ORANGE);
        } else if (sValue.startsWith("FATAL")) {
            setIcon(new ImageIcon(resources.class.getResource(
                    "console_error.png")));
            setBackground(Color.RED);
        }

        return this;
    }
}
