/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * BluemoveApp.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: BluemoveApp.java,v 1.21 2005/02/09 15:18:03 jaquier Exp $
 */


package birg.bluemove.main.gui;

import java.awt.*;
import java.awt.event.*;
import java.net.URL;

import javax.swing.*;
import javax.swing.border.EmptyBorder;

import birg.bluemove.config.gui.PrefsGUI;
import birg.bluemove.logging.gui.Console;
import birg.bluemove.main.core.Bluemove;
import birg.bluemove.module.gui.ModuleTab;
import birg.bluemove.resources.resources;
import birg.bluemove.timelines.gui.TimelinesTab;
import birg.bluemove.utils.gui.About;
import org.apache.log4j.Logger;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: This class is the central GUI part of Bluemove. It contains
 * the menu bar, tool bar, tabbed panes, ...</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 * @author Cyril
 * @version 1.0
 */
public class BluemoveApp extends JFrame {

    static Logger logger = Logger.getLogger(BluemoveApp.class);

    protected static final String m_appName = "Bluemove";
    protected static final String m_version = "1.0";
    protected static final String m_title = m_appName + " v" + m_version;
    protected Bluemove m_bluemove = null;
    ModuleTab m_moduleTab = new ModuleTab();
    TimelinesTab m_timelinesTab = new TimelinesTab();
    Console m_console = new Console();

    JMenuBar jMenuBar1 = new JMenuBar();
    JMenu jMenu1 = new JMenu();
    JMenuItem jMenuItem1 = new JMenuItem();
    JMenu jMenu2 = new JMenu();
    JMenuItem jMenuItem2 = new JMenuItem();
    BorderLayout borderLayout1 = new BorderLayout();
    JToolBar jToolBar1 = new JToolBar();
    JLabel jLabel1 = new JLabel();
    JTabbedPane jTabbedPane1 = new JTabbedPane();
    JMenuItem jMenuItem3 = new JMenuItem();

    /**
     * Constructor. Initializes graphical components and adds the tabs into the
     * tabbed pane.
     *
     * @param bluemove Bluemove
     */
    public BluemoveApp(Bluemove bluemove) {
        m_bluemove = bluemove;

        try {
            jbInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }

        jTabbedPane1.add(m_timelinesTab);
        jTabbedPane1.setTitleAt(0, "Timelines");
        jTabbedPane1.setIconAt(0,
                               new ImageIcon(resources.class.getResource(
                                       "tab_timelines.png")));

        jTabbedPane1.add(m_moduleTab);
        jTabbedPane1.setTitleAt(1, "Modules Manager");
        jTabbedPane1.setIconAt(1,
                               new ImageIcon(resources.class.getResource(
                                       "tab_modulesmanager.png")));

        jTabbedPane1.add(m_console);
        jTabbedPane1.setTitleAt(2, "Console");
        jTabbedPane1.setIconAt(2,
                               new ImageIcon(resources.class.getResource(
                                       "tab_console.png")));

        this.setIconImage(new ImageIcon(resources.class.getResource(
                "icon_bluemove.png")).getImage());

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        pack();
    }

    private JButton m_toolbarButtonNew;
    private JButton m_toolbarButtonOpen;
    private JButton m_toolbarButtonSave;
    private JButton m_toolbarButtonHelp;
    private JButton m_toolbarButtonSplineTool;
    private JButton m_toolbarButtonLinearTool;

    JMenuItem jMenuItem4 = new JMenuItem();
    JMenuItem jMenuItem5 = new JMenuItem();
    JMenuItem jMenuItem6 = new JMenuItem();
    JMenu jMenu3 = new JMenu();
    JMenuItem jMenuItem7 = new JMenuItem();
    JMenuItem jMenuItem8 = new JMenuItem();
    JMenuItem jMenuHist1 = new JMenuItem();

    /**
     * Adds a button to a toolbar. This method was found at
     * http://www.cs.cf.ac.uk/Dave/HCI/HCI_Handout_CALLER/node85.html
     *
     * @param toolBar JToolBar
     * @param bUseImage boolean
     * @param sButtonText String
     * @param imageLocation URL
     * @param sToolHelp String
     * @return JButton
     */
    public JButton addToolbarButton(JToolBar toolBar, boolean bUseImage,
                                    String sButtonText,
                                    URL imageLocation, String sToolHelp) {
        JButton b;

        // Create a new button
        if (bUseImage) {
            b = new JButton(new ImageIcon(imageLocation));
        } else {
            b = (JButton) toolBar.add(new JButton());
        }

        // Add the button to the toolbar
        toolBar.add(b);

        b.setBorder(new EmptyBorder(4, 4, 4, 4));
        // Add optional button text
        if (sButtonText != null) {
            b.setText(sButtonText);
        } else {
            // Only a graphic, so make the button smaller
            b.setMargin(new Insets(0, 0, 0, 0));
        }

        // Add optional tooltip help
        if (sToolHelp != null) {
            b.setToolTipText(sToolHelp);
        }

        // Make sure this button sends a message when the user clicks it
        b.setActionCommand("Toolbar:" + imageLocation);
        b.addActionListener(new BluemoveApp_toolbarButtons_actionAdapter(this));

        // b.addActionListener(this);

        return b;
    }


    /**
     * Initializes graphical components.
     *
     * @throws Exception
     */
    private void jbInit() throws Exception {
        m_toolbarButtonNew = addToolbarButton(jToolBar1, true, "",
                                              resources.class.getResource(
                "toolbar_new.png"), "New project");
        m_toolbarButtonOpen = addToolbarButton(jToolBar1, true, "",
                                               resources.class.getResource(
                "toolbar_open.png"), "Open project");
        m_toolbarButtonSave = addToolbarButton(jToolBar1, true, "",
                                               resources.class.getResource(
                "toolbar_save.png"), "Save project");
        m_toolbarButtonHelp = addToolbarButton(jToolBar1, true, "",
                                               resources.class.getResource(
                "toolbar_help.png"), "Help");

        jToolBar1.addSeparator();

        m_toolbarButtonSplineTool = addToolbarButton(jToolBar1, true, "",
                resources.class.getResource(
                        "toolbar_spline.png"), "Spline tool");

        m_toolbarButtonLinearTool = addToolbarButton(jToolBar1, true, "",
                resources.class.getResource(
                        "toolbar_linear.png"), "Linear tool");

        jToolBar1.setFloatable(false);
        this.setMinimumSize(new Dimension(640, 480));
        this.setJMenuBar(jMenuBar1);
        this.setTitle(m_title);
        this.getContentPane().setLayout(borderLayout1);
        jMenu1.setText("File");
        jMenu1.setMnemonic('F');
        jMenu3.setText("Options");
        jMenu3.setMnemonic('O');
        jMenu2.setText("Help");
        jMenu2.setMnemonic('H');
        jMenuItem2.setText("About");
        jMenuItem2.addActionListener(new BluemoveApp_jMenuItem2_actionAdapter(this));
        jMenuItem8.setText("Help");
        jMenuItem8.setAccelerator(KeyStroke.getKeyStroke("F1"));
        jMenuItem8.addActionListener(new BluemoveApp_jMenuItem8_actionAdapter(this));
        jLabel1.setBorder(BorderFactory.createLoweredBevelBorder());
        jLabel1.setMinimumSize(new Dimension(640, 19));
        jLabel1.setPreferredSize(new Dimension(640, 19));
        jLabel1.setText("Status");
        jTabbedPane1.setDoubleBuffered(true);
        jTabbedPane1.setMinimumSize(new Dimension(640, 480));
        jTabbedPane1.setPreferredSize(new Dimension(640, 480));
        jMenuItem3.setText("Open project...");
        jMenuItem3.setAccelerator(KeyStroke.getKeyStroke('O',
                InputEvent.CTRL_DOWN_MASK));
        jMenuItem3.addActionListener(new BluemoveApp_jMenuItem3_actionAdapter(this));
        jMenuItem4.setText("Save");
        jMenuItem4.setAccelerator(KeyStroke.getKeyStroke('S',
                InputEvent.CTRL_DOWN_MASK));
        jMenuItem4.addActionListener(new BluemoveApp_jMenuItem4_actionAdapter(this));
        jMenuItem1.setText("Exit");
        jMenuItem1.addActionListener(new BluemoveApp_jMenuItem1_actionAdapter(this));
        jMenuItem5.setText("Save As...");
        jMenuItem5.addActionListener(new BluemoveApp_jMenuItem5_actionAdapter(this));
        m_toolbarButtonNew.addActionListener(new
                                             BluemoveApp_m_toolbarButtonNew_actionAdapter(this));
        m_toolbarButtonOpen.addActionListener(new
                                              BluemoveApp_m_toolbarButtonOpen_actionAdapter(this));
        m_toolbarButtonSave.addActionListener(new
                                              BluemoveApp_m_toolbarButtonSave_actionAdapter(this));
        m_toolbarButtonHelp.addActionListener(new
                                              BluemoveApp_m_toolbarButtonHelp_actionAdapter(this));

        m_toolbarButtonSplineTool.addActionListener(new
                BluemoveApp_m_toolbarButtonSplineTool_actionAdapter(this));

        m_toolbarButtonLinearTool.addActionListener(new
                BluemoveApp_m_toolbarButtonLinearTool_actionAdapter(this));

        jMenuItem6.setText("New");
        jMenuItem6.setAccelerator(KeyStroke.getKeyStroke('N',
                InputEvent.CTRL_DOWN_MASK));
        jMenuItem6.addActionListener(new BluemoveApp_jMenuItem6_actionAdapter(this));
        jMenuItem7.setText("Preferences...");
        jMenuItem7.setAccelerator(KeyStroke.getKeyStroke('P',
                InputEvent.CTRL_DOWN_MASK));
        jMenuItem7.addActionListener(new BluemoveApp_jMenuItem7_actionAdapter(this));

        jMenuBar1.add(jMenu1);
        jMenuBar1.add(jMenu3);
        jMenuBar1.add(jMenu2);
        jMenu1.add(jMenuItem6);
        jMenu1.add(jMenuItem3);
        jMenu1.add(jMenuItem4);
        jMenu1.add(jMenuItem5);
        jMenu1.addSeparator();

        // Project history
        this.updateProjectHistory();
        jMenuHist1.addActionListener(new ProjectHistory_MenuItem_actionAdapter(this));
        jMenu1.add(jMenuHist1);

        jMenu1.addSeparator();
        jMenu1.add(jMenuItem1);
        jMenu2.add(jMenuItem8);
        jMenu2.addSeparator();
        jMenu2.add(jMenuItem2);
        jMenu3.add(jMenuItem7);
        this.getContentPane().add(jTabbedPane1, java.awt.BorderLayout.CENTER);
        this.getContentPane().add(jToolBar1, java.awt.BorderLayout.NORTH);
        this.getContentPane().add(jLabel1, java.awt.BorderLayout.SOUTH);
    }

    /**
     * Updates the project history menu.
     */
    public void updateProjectHistory() {
        // Project history
        String history[][] = m_bluemove.getProjectHistory();
        jMenuHist1.setText("1 " + history[0][0]);
        jMenuHist1.setActionCommand(history[0][1]);
    }

    /**
     * Sets the Bluemove's window title.
     *
     * @param title String
     */
    public void setAppTitle(String title) {
        this.setTitle(m_title + " - " + title);
    }

    /**
     * Action performed on the exit menu item.
     *
     * @param e ActionEvent
     */
    public void menuItemProjectHistory_actionPerformed(ActionEvent e) {
        m_bluemove.openProject(e.getActionCommand());
        m_moduleTab.initialize();
    }

    /**
     * Action performed on the exit menu item.
     *
     * @param e ActionEvent
     */
    public void jMenuItem1_actionPerformed(ActionEvent e) {
        System.exit(0);
    }

    /**
     * Action performed on the toolbar.
     *
     * @param event ActionEvent
     */
    public void toolbarButtons_actionPerformed(ActionEvent event) {
        logger.debug("Action performed on toolbar");
    }

    /**
     * Action performed on the <pre>open</pre> menu item.
     *
     * @param e ActionEvent
     */
    public void jMenuItem3_actionPerformed(ActionEvent e) {
        m_bluemove.openProject();
        m_moduleTab.initialize();
        this.updateProjectHistory();
    }

    /**
     * Action performed on the <pre>save</pre> menu item.
     *
     * @param e ActionEvent
     */
    public void jMenuItem4_actionPerformed(ActionEvent e) {
        m_bluemove.saveProject();
    }

    /**
     * Action performed on the <pre>save as</pre> menu item.
     *
     * @param e ActionEvent
     */
    public void jMenuItem5_actionPerformed(ActionEvent e) {
        m_bluemove.saveProjectAs();
        this.updateProjectHistory();
    }

    /**
     * Action performed on the toolbar button <pre>new project</pre>.
     *
     * @param e ActionEvent
     */
    public void m_toolbarButtonNew_actionPerformed(ActionEvent e) {
        m_bluemove.newProject();
    }

    /**
     * Action performed on the toolbar button <pre>open project</pre>.
     *
     * @param e ActionEvent
     */
    public void m_toolbarButtonOpen_actionPerformed(ActionEvent e) {
        m_bluemove.openProject();
        m_moduleTab.initialize();
        this.updateProjectHistory();
    }

    /**
     * Action performed on the toolbar button <pre>save project</pre>.
     *
     * @param e ActionEvent
     */
    public void m_toolbarButtonSave_actionPerformed(ActionEvent e) {
        m_bluemove.saveProject();
    }

    /**
     * Action performed on the toolbar button <pre>help</pre>.
     *
     * @param e ActionEvent
     */
    public void m_toolbarButtonHelp_actionPerformed(ActionEvent e) {
        m_bluemove.displayHelp();
    }

    /**
     * Action performed on the spline tool button <pre>help</pre>.
     *
     * @param e ActionEvent
     */
    public void m_toolbarButtonSplineTool_actionPerformed(ActionEvent e) {
        m_timelinesTab.m_currentToolType = TimelinesTab.ToolType.ADD_SPLINE;
    }

    /**
     * Action performed on the linear tool button <pre>help</pre>.
     *
     * @param e ActionEvent
     */
    public void m_toolbarButtonLinearTool_actionPerformed(ActionEvent e) {
        m_timelinesTab.m_currentToolType = TimelinesTab.ToolType.ADD_LINEAR;
    }

    /**
     * Action performed on the menu item <pre>new project</pre>.
     *
     * @param e ActionEvent
     */
    public void jMenuItem6_actionPerformed(ActionEvent e) {
        m_bluemove.newProject();
    }

    /**
     * Action performed on the menu item <pre>preferences</pre>.
     *
     * @param e ActionEvent
     */
    public void jMenuItem7_actionPerformed(ActionEvent e) {
        PrefsGUI prefs = new PrefsGUI();
        prefs.setSize(400, 320);
        prefs.setLocationRelativeTo(this);
        prefs.setModal(true);
        prefs.setVisible(true);
    }

    /**
     * Action performed on the menu item <pre>help</pre>.
     *
     * @param e ActionEvent
     */
    public void jMenuItem8_actionPerformed(ActionEvent e) {
        m_bluemove.displayHelp();
    }

    /**
     * Action performed on the menu item <pre>about</pre>.
     *
     * @param e ActionEvent
     */
    public void jMenuItem2_actionPerformed(ActionEvent e) {
        About about = new About();
        about.setLocationRelativeTo(this);
        about.setModal(true);
        about.setVisible(true);
    }
}


class BluemoveApp_m_toolbarButtonNew_actionAdapter implements ActionListener {
    private BluemoveApp adaptee;
    BluemoveApp_m_toolbarButtonNew_actionAdapter(BluemoveApp adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.m_toolbarButtonNew_actionPerformed(e);
    }
}


class BluemoveApp_m_toolbarButtonOpen_actionAdapter implements ActionListener {
    private BluemoveApp adaptee;
    BluemoveApp_m_toolbarButtonOpen_actionAdapter(BluemoveApp adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.m_toolbarButtonOpen_actionPerformed(e);
    }
}


class BluemoveApp_m_toolbarButtonSave_actionAdapter implements ActionListener {
    private BluemoveApp adaptee;
    BluemoveApp_m_toolbarButtonSave_actionAdapter(BluemoveApp adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.m_toolbarButtonSave_actionPerformed(e);
    }
}


class BluemoveApp_m_toolbarButtonHelp_actionAdapter implements ActionListener {
    private BluemoveApp adaptee;
    BluemoveApp_m_toolbarButtonHelp_actionAdapter(BluemoveApp adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.m_toolbarButtonHelp_actionPerformed(e);
    }
}


class ProjectHistory_MenuItem_actionAdapter implements ActionListener {
    private BluemoveApp adaptee;
    ProjectHistory_MenuItem_actionAdapter(BluemoveApp adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.menuItemProjectHistory_actionPerformed(e);
    }
}


class BluemoveApp_jMenuItem5_actionAdapter implements ActionListener {
    private BluemoveApp adaptee;
    BluemoveApp_jMenuItem5_actionAdapter(BluemoveApp adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.jMenuItem5_actionPerformed(e);
    }
}


class BluemoveApp_jMenuItem4_actionAdapter implements ActionListener {
    private BluemoveApp adaptee;
    BluemoveApp_jMenuItem4_actionAdapter(BluemoveApp adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.jMenuItem4_actionPerformed(e);
    }
}


class BluemoveApp_jMenuItem3_actionAdapter implements ActionListener {
    private BluemoveApp adaptee;
    BluemoveApp_jMenuItem3_actionAdapter(BluemoveApp adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.jMenuItem3_actionPerformed(e);
    }
}


class BluemoveApp_jMenuItem6_actionAdapter implements ActionListener {
    private BluemoveApp adaptee;
    BluemoveApp_jMenuItem6_actionAdapter(BluemoveApp adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.jMenuItem6_actionPerformed(e);
    }
}


class BluemoveApp_jMenuItem7_actionAdapter implements ActionListener {
    private BluemoveApp adaptee;
    BluemoveApp_jMenuItem7_actionAdapter(BluemoveApp adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.jMenuItem7_actionPerformed(e);
    }
}


class BluemoveApp_jMenuItem8_actionAdapter implements ActionListener {
    private BluemoveApp adaptee;
    BluemoveApp_jMenuItem8_actionAdapter(BluemoveApp adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.jMenuItem8_actionPerformed(e);
    }
}


class BluemoveApp_jMenuItem2_actionAdapter implements ActionListener {
    private BluemoveApp adaptee;
    BluemoveApp_jMenuItem2_actionAdapter(BluemoveApp adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.jMenuItem2_actionPerformed(e);
    }
}


class BluemoveApp_jMenuItem1_actionAdapter implements ActionListener {
    private BluemoveApp adaptee;
    BluemoveApp_jMenuItem1_actionAdapter(BluemoveApp adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.jMenuItem1_actionPerformed(e);
    }
}


class BluemoveApp_toolbarButtons_actionAdapter implements ActionListener {
    private BluemoveApp adaptee;
    BluemoveApp_toolbarButtons_actionAdapter(BluemoveApp adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.toolbarButtons_actionPerformed(e);
    }
}


class BluemoveApp_m_toolbarButtonSplineTool_actionAdapter implements
        ActionListener {
    private BluemoveApp adaptee;
    BluemoveApp_m_toolbarButtonSplineTool_actionAdapter(BluemoveApp adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.m_toolbarButtonSplineTool_actionPerformed(e);
    }
}


class BluemoveApp_m_toolbarButtonLinearTool_actionAdapter implements
        ActionListener {
    private BluemoveApp adaptee;
    BluemoveApp_m_toolbarButtonLinearTool_actionAdapter(BluemoveApp adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.m_toolbarButtonLinearTool_actionPerformed(e);
    }
}
