/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * Actuator.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: Actuator.java,v 1.6 2005/01/25 23:47:13 jaquier Exp $
 */

package birg.bluemove.module.core;

import birg.bluemove.timelines.core.curve.ActuatorCurve;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: This class represents an actuator (servo, motor, ...). It
 * inherits from <pre>ModuleComponent</pre> as the <pre>Sensor</pre> class
 * do. It has a reference to his parent module.</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 * @author Cyril
 * @version 1.0
 */
public class Actuator extends ModuleComponent {

    private static Integer m_counter = 1;
    private double m_rangeMin = 0.0;
    private double m_rangeMax = 0.0;

    /**
     * Constructor. The param <pre>module</pre> is the parent module.
     *
     * @param module Module
     * @throws Exception
     */
    public Actuator(Module module) throws Exception {
        this(module, "Actuator" + m_counter);
    }

    /**
     * Constructor. The param <pre>module</pre> is the parent module.
     *
     * @param module Module
     * @throws Exception
     */
    public Actuator(Module module, String name) throws Exception {
        super(module, name);
        m_curve = new ActuatorCurve(this);
        m_counter++;
    }

    /**
     * Gets the minimum value allowed for the actuator.
     *
     * @return double
     */
    public double getRangeMin() {
        return m_rangeMin;
    }

    /**
     * Gets the maximum value allowed for the actuator.
     *
     * @return double
     */
    public double getRangeMax() {
        return m_rangeMax;
    }

    /**
     * Sets the range using a <pre>double</pre> array.
     *
     * @param range Double[]
     * @throws Exception
     */
    public void setRange(double[] range) throws Exception {
        setRange(range[0], range[1]);
    }

    /**
     * Sets the range. If min is greater than max, throws an exception.
     *
     * @param min double
     * @param max double
     * @throws Exception
     * @todo Use a specific exception.
     */
    public void setRange(double min, double max) throws Exception {
        if (min <= max) {
            m_rangeMin = min;
            m_rangeMax = max;
        } else {
            throw new Exception("min must be smaller than max.");
        }
    }
}
