/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * ModuleComponent.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: ModuleComponent.java,v 1.6 2005/01/25 23:47:13 jaquier Exp $
 */

package birg.bluemove.module.core;

import birg.bluemove.timelines.core.curve.Curve;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: This class represents a module component like an actuator or
 * a sensor. This class should not be used directly but should be inherited.</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 * @author Cyril
 * @version 1.0
 */
public class ModuleComponent {

    protected String m_name = null;
    protected Curve m_curve = null;
    protected Module m_module = null;

    /**
     * Constructor. Sets the module reference and the component name.
     *
     * @param module Module
     * @param name String
     * @throws Exception
     */
    public ModuleComponent(Module module, String name) throws Exception {
        setModule(module);
        setName(name);
    }

    /**
     * Constructor. Sets the module reference.
     *
     * @param module Module
     * @throws Exception
     */
    public ModuleComponent(Module module) throws Exception {
        this(module, null);
    }

    /**
     * Returns the parent module of the component.
     *
     * @return Module
     */
    public Module getModule() {
        return m_module;
    }

    /**
     * Sets the parent module of the component.
     *
     * @param module Module
     */
    public void setModule(Module module) {
        m_module = module;
    }

    /**
     * Gets the name of the component.
     *
     * @return String
     */
    public String getName() {
        return m_name;
    }

    /**
     * Sets the name of the component. Throws an exception if the name already
     * exists for a component in this module.
     *
     * @param name String
     * @throws Exception
     * @todo Use a specific exception.
     */
    public void setName(String name) throws Exception {
        ModuleComponent temp = m_module.getComponent(name);
        if (temp != null) {
            throw new Exception("Duplicate component name " + name);
        }
        m_name = name;
    }

    /**
     * Gets the curve of the component.
     *
     * @return Curve
     */
    public Curve getCurve() {
        return m_curve;
    }

    /**
     * Sets the curve of the component.
     *
     * @param curve Curve
     */
    public void setCurve(Curve curve) {
        m_curve = curve;
    }

    /**
     * Returns the string representation of a component. Simply returns the
     * component name.
     *
     * @return String
     */
    public String toString() {
        return m_name;
    }
}
