/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * ActuatorSetting.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: ActuatorSetting.java,v 1.7 2005/01/25 23:46:44 jaquier Exp $
 */

package birg.bluemove.module.gui;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import birg.bluemove.module.core.Actuator;
import birg.bluemove.resources.resources;
import birg.bluemove.utils.gui.MessageBox;
import org.apache.log4j.Logger;

public class ActuatorSetting extends JPanel implements Setting, ActionListener {

    static Logger logger = Logger.getLogger(ActuatorSetting.class);

    ModuleTab m_mTab = null;
    private Actuator m_actuator = null;

    JScrollPane jScrollPane1 = new JScrollPane();
    JPanel jPanel1 = new JPanel();
    BorderLayout borderLayout1 = new BorderLayout();
    JPanel jPanel2 = new JPanel();
    GridBagLayout gridBagLayout1 = new GridBagLayout();
    JPanel jPanel3 = new JPanel();
    JTextField jTextField1 = new JTextField();
    JLabel jLabel1 = new JLabel();
    JPanel jPanel4 = new JPanel();
    JLabel jLabel2 = new JLabel();
    JSpinner jSpinner1 = new JSpinner();
    JPanel jPanel5 = new JPanel();
    JLabel jLabel4 = new JLabel();
    JSpinner jSpinner2 = new JSpinner();
    SpinnerNumberModel spinModel1 = new SpinnerNumberModel();
    SpinnerNumberModel spinModel2 = new SpinnerNumberModel();
    JButton applyButton = new JButton();
    BorderLayout borderLayout2 = new BorderLayout();
    JPanel jPanel6 = new JPanel();

    ActuatorSetting_jSpinner1_changeAdapter jSpinner1Adapter = null;
    ActuatorSetting_jSpinner2_changeAdapter jSpinner2Adapter = null;

    public ActuatorSetting(ModuleTab mTab) {
        m_mTab = mTab;
        try {
            jbInit();
        } catch (Exception ex) {
            logger.error(ex.getMessage());
        }
    }

    private void jbInit() throws Exception {
        this.setLayout(borderLayout2);
        applyButton.setText("Apply");
        applyButton.setIcon(new ImageIcon(resources.class.getResource(
                "icon_accept.png")));
        applyButton.setActionCommand(BUTTON_APPLY);
        applyButton.addActionListener(this);
        applyButton.setEnabled(false);
        this.add(jPanel6, java.awt.BorderLayout.SOUTH);
        jPanel6.add(applyButton);

        jPanel1.setLayout(borderLayout1);
        jPanel2.setLayout(gridBagLayout1);
        jTextField1.setText("Actuator name");
        jTextField1.setColumns(12);
        jTextField1.addKeyListener(new ActuatorSetting_jTextArea1_keyAdapter(this));
        jLabel1.setPreferredSize(new Dimension(95, 15));
        jLabel1.setToolTipText("");
        jLabel1.setText("Actuator name");
        jLabel2.setPreferredSize(new Dimension(95, 15));
        jLabel2.setText("Min value");
        jSpinner1.setPreferredSize(new Dimension(136, 20));
        jSpinner1Adapter = new ActuatorSetting_jSpinner1_changeAdapter(this);
        jLabel4.setPreferredSize(new Dimension(95, 15));
        jLabel4.setText("Max value");
        jSpinner2.setPreferredSize(new Dimension(136, 20));
        jSpinner2Adapter = new ActuatorSetting_jSpinner2_changeAdapter(this);
        this.add(jScrollPane1, java.awt.BorderLayout.CENTER);
        jScrollPane1.getViewport().add(jPanel1);
        jPanel1.add(jPanel2, java.awt.BorderLayout.NORTH);
        jPanel3.add(jLabel1);
        jPanel3.add(jTextField1);
        jPanel4.add(jLabel2);
        jPanel4.add(jSpinner1);

        jPanel2.add(jPanel5, new GridBagConstraints(0, 2, 1, 1, 1.0, 1.0
                , GridBagConstraints.WEST, GridBagConstraints.NONE,
                new Insets(0, 0, 0, 0), 0, 0));
        jPanel5.add(jLabel4);
        jPanel5.add(jSpinner2);
        jPanel2.add(jPanel3, new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0
                , GridBagConstraints.WEST, GridBagConstraints.NONE,
                new Insets(0, 0, 0, 0), 0, 0));
        jPanel2.add(jPanel4, new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0
                , GridBagConstraints.WEST, GridBagConstraints.NONE,
                new Insets(0, 0, 0, 0), 0, 0));

        spinModel1.setStepSize(0.01);
        spinModel1.setValue(0.0);
        spinModel2.setStepSize(0.01);
        spinModel2.setValue(0.0);
        jSpinner1.setModel(spinModel1);
        jSpinner2.setModel(spinModel2);
    }

    private void setActuatorValues() {
        jTextField1.setText(m_actuator.getName());

        // Remove the change listener to avoid the firing of an
        // event when changing the actuator
        jSpinner1.removeChangeListener(jSpinner1Adapter);
        jSpinner2.removeChangeListener(jSpinner2Adapter);

        jSpinner1.setValue(m_actuator.getRangeMin());
        jSpinner2.setValue(m_actuator.getRangeMax());

        // Reinstall the change listener
        jSpinner1.addChangeListener(jSpinner1Adapter);
        jSpinner2.addChangeListener(jSpinner2Adapter);
    }

    public void loadSettings(Object obj) {
        m_actuator = (Actuator) obj;
        setActuatorValues();
    }

    public void saveSettings() {
        String str = null;

        str = jTextField1.getText();
        try {
            if (!str.equals(m_actuator.getName())) {
                m_actuator.setName(str);
            }
        } catch (Exception e) {
            logger.error(e.getMessage());
            String msg = "Unable to create a component with this name. " +
                         "Name already exits.";
            MessageBox mBox = new MessageBox(null,
                                             MessageBox.MessageBoxType.ERROR,
                                             msg);
            mBox.showDialog();
        }

        Double min = (Double) spinModel1.getNumber();
        Double max = (Double) spinModel2.getNumber();
        if (min != m_actuator.getRangeMin() || max != m_actuator.getRangeMax()) {
            try {
                m_actuator.setRange(min, max);
            } catch (Exception e) {
                logger.error(e.getMessage());
                String msg = "Min (" + min + ") must be smaller than max (" +
                             max + ").";
                MessageBox mBox = new MessageBox(null,
                                                 MessageBox.MessageBoxType.
                                                 ERROR,
                                                 msg);
                mBox.showDialog();
            }
        }
    }

    public void actionPerformed(ActionEvent e) {
        if (BUTTON_APPLY.equals(e.getActionCommand())) {
            saveSettings();
            m_mTab.moduleTree.refreshTree();
            applyButton.setEnabled(false);
        }
    }

    public void jTextArea1_keyTyped(KeyEvent e) {
        applyButton.setEnabled(true);
    }

    public void jSpinner1_stateChanged(ChangeEvent e) {
        applyButton.setEnabled(true);
    }

    public void jSpinner2_stateChanged(ChangeEvent e) {
        applyButton.setEnabled(true);
    }
}


class ActuatorSetting_jTextArea1_keyAdapter extends KeyAdapter {
    private ActuatorSetting adaptee;
    ActuatorSetting_jTextArea1_keyAdapter(ActuatorSetting adaptee) {
        this.adaptee = adaptee;
    }

    public void keyTyped(KeyEvent e) {
        adaptee.jTextArea1_keyTyped(e);
    }
}


class ActuatorSetting_jSpinner1_changeAdapter implements ChangeListener {
    private ActuatorSetting adaptee;
    ActuatorSetting_jSpinner1_changeAdapter(ActuatorSetting adaptee) {
        this.adaptee = adaptee;
    }

    public void stateChanged(ChangeEvent e) {
        adaptee.jSpinner1_stateChanged(e);
    }
}


class ActuatorSetting_jSpinner2_changeAdapter implements ChangeListener {
    private ActuatorSetting adaptee;
    ActuatorSetting_jSpinner2_changeAdapter(ActuatorSetting adaptee) {
        this.adaptee = adaptee;
    }

    public void stateChanged(ChangeEvent e) {
        adaptee.jSpinner2_stateChanged(e);
    }
}
