/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * ModuleTab.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: ModuleTab.java,v 1.6 2005/01/16 11:47:05 jaquier Exp $
 */

package birg.bluemove.module.gui;

import java.awt.BorderLayout;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JPanel;
import javax.swing.JSplitPane;

import birg.bluemove.module.core.*;
import org.apache.log4j.Logger;

public class ModuleTab extends JPanel implements Observer {

    static Logger logger = Logger.getLogger(ModuleTab.class);
    private ModulesManager m_mManager = ModulesManager.getInstance();

    JSplitPane jSplitPane1 = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
    BorderLayout borderLayout1 = new BorderLayout();

    ModuleTree moduleTree = new ModuleTree(this);
    ModuleSetting moduleSetting = new ModuleSetting(this);
    ProjectSetting projectSetting = new ProjectSetting();
    ActuatorSetting actuatorSetting = new ActuatorSetting(this);
    SensorSetting sensorSetting = new SensorSetting(this);

    public ModuleTab() {
        m_mManager.addObserver(this);
        try {
            jbInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    private void jbInit() throws Exception {
        jSplitPane1.setOneTouchExpandable(true);
        jSplitPane1.add(projectSetting, JSplitPane.RIGHT);
        jSplitPane1.add(moduleTree, JSplitPane.LEFT);
        this.setLayout(borderLayout1);
        this.add(jSplitPane1, java.awt.BorderLayout.CENTER);
    }

    public void setSettingPanel(Object obj) {
        if (obj instanceof Module) {
            jSplitPane1.setRightComponent(moduleSetting);
            moduleSetting.loadSettings(obj);
        } else if (obj instanceof Actuator) {
            jSplitPane1.setRightComponent(actuatorSetting);
            actuatorSetting.loadSettings(obj);
        } else if (obj instanceof Sensor) {
            jSplitPane1.setRightComponent(sensorSetting);
            sensorSetting.loadSettings(obj);
        } else {
            jSplitPane1.setRightComponent(projectSetting);
            projectSetting.loadSettings(m_mManager);
        }
    }

    public void initialize() {
        moduleTree.createInitialTree();
        setSettingPanel("");
    }

    public void update(Observable o, Object arg) {
        logger.debug("Update observer --> " + this.getClass().getName());
        if (o instanceof ModulesManager) {
            if (arg.toString().equals("clear")) {
                moduleTree.createInitialTree();
                setSettingPanel(null);
                projectSetting.clear();
            }
        }
    }
}
