/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * ModuleTree.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: ModuleTree.java,v 1.12 2005/01/24 23:49:59 jaquier Exp $
 */

package birg.bluemove.module.gui;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.*;

import birg.bluemove.communication.gui.ModulesListGUI;
import birg.bluemove.module.core.*;
import birg.bluemove.resources.resources;
import birg.bluemove.utils.gui.MessageBox;
import org.apache.log4j.Logger;

public class ModuleTree extends JPanel implements TreeSelectionListener,
        ActionListener {

    static Logger logger = Logger.getLogger(ModuleTree.class);

    ModulesListGUI m_gui = null;
    private ModulesManager m_mManager = ModulesManager.getInstance();
    private DefaultTreeModel m_treeModel = null;
    private ModuleTab m_moduleTab = null;

    DefaultMutableTreeNode m_top = new DefaultMutableTreeNode(NodeType.PROJECT);

    JTree jTree1 = null;
    JPanel jPanel1 = new JPanel();
    JButton addButton = new JButton();
    BorderLayout borderLayout1 = new BorderLayout();
    JButton delButton = new JButton();
    JScrollPane jScrollPane1 = new JScrollPane();

    protected enum NodeType {
        PROJECT("Project"), ACTUATORS("Actuators"), SENSORS("Sensors");

        private String m_name = null;
        NodeType(String name) {
            m_name = name;
        }

        public String toString() {
            return m_name;
        }
    }


    private static final String BUTTON_ADD = "add";
    private static final String BUTTON_DEL = "del";

    public ModuleTree(ModuleTab moduleTab) {
        m_moduleTab = moduleTab;

        m_treeModel = new DefaultTreeModel(m_top);
        jTree1 = new JTree(m_treeModel);
        jTree1.getSelectionModel().setSelectionMode(TreeSelectionModel.
                SINGLE_TREE_SELECTION);
        //Listen for when the selection changes.
        jTree1.addTreeSelectionListener(this);
        jTree1.setCellRenderer(new BluemoveTreeCellRenderer());
        jTree1.setRowHeight(20);

        try {
            jbInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void createInitialTree() {
        // Start with a new tree
        m_top.removeAllChildren();
        for (Module element : m_mManager.getModuleList()) {
            DefaultMutableTreeNode node[] = addModuleNode(element);
            for (ModuleComponent comp : element.getComponentList()) {
                if (comp instanceof Actuator) {
                    addComponentNode(node[1], comp);
                } else if (comp instanceof Sensor) {
                    addComponentNode(node[2], comp);
                }
            }
        }
        m_treeModel.reload();
    }

    private DefaultMutableTreeNode[] addModuleNode(Module module) {
        DefaultMutableTreeNode newNode = null;
        DefaultMutableTreeNode actu = null;
        DefaultMutableTreeNode sens = null;

        newNode = new DefaultMutableTreeNode(module);
        // Insert the node at the bottom of the tree
        m_treeModel.insertNodeInto(newNode, m_top,
                                   m_treeModel.getChildCount(m_top));
        actu = new DefaultMutableTreeNode(NodeType.ACTUATORS);
        newNode.add(actu);
        sens = new DefaultMutableTreeNode(NodeType.SENSORS);
        newNode.add(sens);

        DefaultMutableTreeNode[] nodeArray = {newNode, actu, sens};
        return nodeArray;
    }

    private DefaultMutableTreeNode addComponentNode(DefaultMutableTreeNode node,
            ModuleComponent comp) {
        DefaultMutableTreeNode newNode = null;
        newNode = new DefaultMutableTreeNode(comp);
        // Insert the node at the bottom of the tree
        m_treeModel.insertNodeInto(newNode, node,
                                   m_treeModel.getChildCount(node));
        return newNode;
    }

    protected void addNode(DefaultMutableTreeNode node) {
        DefaultMutableTreeNode newNode = null;
        if (node.isRoot()) {
            try {
                Module m = new Module();
                m_mManager.addModule(m);
                newNode = addModuleNode(m)[0];
            } catch (Exception e) {
                logger.error(e.getMessage());
                String msg = "Unable to create a module with this name. " +
                             "Name already exits.";
                MessageBox mBox = new MessageBox(null,
                                                 MessageBox.MessageBoxType.
                                                 ERROR,
                                                 msg);
                mBox.showDialog();
            }
        } else if (node.getUserObject() instanceof NodeType) {
            NodeType value = (NodeType) node.getUserObject();
            Module module = (Module) ((DefaultMutableTreeNode) node.getParent()).
                            getUserObject();
            try {
                ModuleComponent component = null;
                if (value == NodeType.ACTUATORS) {
                    component = new Actuator(module);
                } else if (value == NodeType.SENSORS) {
                    component = new Sensor(module);
                }
                module.addComponent(component);
                newNode = addComponentNode(node, component);
            } catch (Exception e) {
                logger.error(e.getMessage());
                String msg = "Unable to create a component with this name. " +
                             "Name already exits.";
                MessageBox mBox = new MessageBox(null,
                                                 MessageBox.MessageBoxType.
                                                 ERROR,
                                                 msg);
                mBox.showDialog();
            }
        }

        if (newNode != null) {
            jTree1.scrollPathToVisible(new TreePath(newNode.getPath()));
        }
    }

    protected void deleteNode(DefaultMutableTreeNode node) {
        Object selectedNode = node.getUserObject();
        if (selectedNode instanceof Module) {
            m_mManager.removeModule((Module) selectedNode);
            m_treeModel.removeNodeFromParent(node);
        } else if (selectedNode instanceof ModuleComponent) {
            ((Module) ((DefaultMutableTreeNode) node.getParent().getParent()).
             getUserObject()).removeComponent((ModuleComponent) selectedNode);
            m_treeModel.removeNodeFromParent(node);
        }
    }

    public void refreshTree() {
        TreeNode current = (TreeNode) jTree1.getLastSelectedPathComponent();
        m_treeModel.nodeChanged(current);
    }

    public void valueChanged(TreeSelectionEvent e) {
        DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                                      jTree1.getLastSelectedPathComponent();

        if (node == null) {
            return;
        }

        Object nodeInfo = node.getUserObject();
        m_moduleTab.setSettingPanel(nodeInfo);

        if (nodeInfo instanceof Module) {
            addButton.setEnabled(false);
            delButton.setEnabled(true);
        } else if (nodeInfo instanceof Actuator) {
            addButton.setEnabled(false);
            delButton.setEnabled(true);
        } else if (nodeInfo instanceof Sensor) {
            addButton.setEnabled(false);
            delButton.setEnabled(true);
        } else if (nodeInfo instanceof NodeType) {
            addButton.setEnabled(true);
            delButton.setEnabled(false);
        }
    }

    public void actionPerformed(ActionEvent e) {
        DefaultMutableTreeNode node = null;
        node = (DefaultMutableTreeNode) jTree1.getLastSelectedPathComponent();

        if (node == null) {
            return;
        }

        if (BUTTON_DEL.equals(e.getActionCommand())) {
            deleteNode(node);
            m_moduleTab.setSettingPanel(node.getParent());
        } else if (BUTTON_ADD.equals(e.getActionCommand())) {
            addNode(node);
        }
    }


    private void jbInit() throws Exception {
        this.setLayout(borderLayout1);
        addButton.setText("Add");
        addButton.setIcon(new ImageIcon(resources.class.getResource(
                "icon_add.png")));
        addButton.setActionCommand(BUTTON_ADD);
        addButton.addActionListener(this);
        delButton.setText("Delete");
        delButton.setIcon(new ImageIcon(resources.class.getResource(
                "icon_remove.png")));
        delButton.setActionCommand(BUTTON_DEL);
        delButton.addActionListener(this);
        jScrollPane1.setDoubleBuffered(true);
        jPanel1.add(addButton);
        jPanel1.add(delButton);
        jScrollPane1.getViewport().add(jTree1);
        this.add(jPanel1, java.awt.BorderLayout.SOUTH);
        this.add(jScrollPane1, java.awt.BorderLayout.CENTER);
        addButton.setEnabled(false);
        delButton.setEnabled(false);
        jTree1.setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));
    }
}


class BluemoveTreeCellRenderer extends DefaultTreeCellRenderer {
    public Component getTreeCellRendererComponent(JTree tree, Object value,
                                                  boolean sel, boolean expanded,
                                                  boolean leaf, int row,
                                                  boolean hasFocus) {

        super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf,
                                           row, hasFocus);

        Object node = ((DefaultMutableTreeNode) value).getUserObject();

        Font boldFont = getFont().deriveFont(Font.BOLD);
        Font italicFont = getFont().deriveFont(Font.ITALIC);
        setFont(getFont().deriveFont(Font.PLAIN));

        if (node instanceof Module) {
            setIcon(new ImageIcon(resources.class.getResource(
                    "icon_address.png")));
            setFont(boldFont);
        } else if (node instanceof Actuator) {
            setIcon(new ImageIcon(resources.class.getResource(
                    "icon_actuator.png")));
            setFont(boldFont);
        } else if (node instanceof Sensor) {
            setIcon(new ImageIcon(resources.class.getResource("icon_sensor.png")));
            setFont(boldFont);
        } else {
            ModuleTree.NodeType type = (ModuleTree.NodeType) node;
            if (type == ModuleTree.NodeType.PROJECT) {
                setIcon(new ImageIcon(resources.class.getResource(
                        "icon_bluemove.png")));
            } else if (type == ModuleTree.NodeType.ACTUATORS) {
                setIcon(new ImageIcon(resources.class.getResource(
                        "icon_actuator.png")));
                setFont(italicFont);
            } else if (type == ModuleTree.NodeType.SENSORS) {
                setIcon(new ImageIcon(resources.class.getResource(
                        "icon_sensor.png")));
                setFont(italicFont);
            }
        }
        return this;
    }

}
