/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * ActuatorCurve.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: ActuatorCurve.java,v 1.4 2005/01/26 22:41:43 drapel Exp $
 */

package birg.bluemove.timelines.core.curve;

import birg.bluemove.module.core.Actuator;
import birg.bluemove.module.core.ModuleComponent;

/**
 *
 * <p>Title: Bluemove</p>
 *
 * <p>Description: Modular Robot Project</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 *
 * @author Kevin Drapel
 * @version 1.0
 */
public class ActuatorCurve extends Curve {

    protected boolean m_isActive = true;

    /**
     * Constructor.
     * @param comp ModuleComponent
     */
    public ActuatorCurve(ModuleComponent comp) {
        super(comp);
    }

    /**
     * Returns a boolean indicating whetever the curve is activated or not.
     * @return boolean
     */
    public boolean isActive() {
        return m_isActive;
    }

    /**
     * Sets the active state of curve.
     * @param value boolean
     */
    public void setActive(boolean value) {
        m_isActive = value;
    }

    /**
     * Evaluates the curve with clipping and exact range of actuator
     * @param value double
     * @return double
     */
    public double evaluateRealValue(double value) {
        double y = evaluate(value);

        Actuator comp = (Actuator)this.getModuleComponent();

        y = Math.min(y, 1.0);
        y = Math.max(y, 0.0);

        double min = comp.getRangeMin();
        double max = comp.getRangeMax();

        // Interpolation
        return (min + ((max - min) * y));
    }
}
