/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ |/ / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * InterpolatorLinear.java
 *
 * Created on 10. novembre 2004, 23:16
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: InterpolatorLinear.java,v 1.3 2005/01/26 22:41:43 drapel Exp $
 */

package birg.bluemove.timelines.core.interpolator;

import java.util.List;

/**
 *
 * <p>Title: Bluemove</p>
 *
 * <p>Description: Modular Robot Project</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 *
 * @author Kevin Drapel
 * @version $Id: InterpolatorLinear.java,v 1.3 2005/01/26 22:41:43 drapel Exp $
 */
public class InterpolatorLinear implements Interpolator {

    /** Creates a new instance of InterpolatorLinear */
    public InterpolatorLinear() {
    }

    /**
     * Evaluates y=f(x) with x = frame and y is the vertical position value on
     * the segment defined by the two keys
     * @param keys list with exactly two keys (start/end)
     * @param frame "x" value in f(x)
     * @return double "y" value
     */
    public double evaluate(List<Key> keys, double frame) {
        assert (keys.size() == 2);

        // get reference to linear keys
        Key k1 = (Key) keys.get(0);
        Key k2 = (Key) keys.get(1);

        // frames numbers must be consistent
        assert (k1.getFrame() < k2.getFrame());

        // compute difference on frame axis
        double df = k2.getFrame() - k1.getFrame();
        if (df < Double.MIN_VALUE) {
            return k1.getValue();
        }

        // compute difference on value axis
        double dv = k2.getValue() - k1.getValue();

        // get slope
        dv /= df;

        // return interpolated value according to frame
        return k1.getValue() + dv * (frame - k1.getFrame());

    }

}
