/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * TimelinesManager.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: TimelinesManager.java,v 1.6 2005/01/27 16:32:34 jaquier Exp $
 */

package birg.bluemove.timelines.core.timeline;

import java.util.prefs.Preferences;

import birg.bluemove.main.core.Bluemove;
import birg.bluemove.module.core.*;
import birg.bluemove.timelines.core.interpolator.Key;
import org.apache.log4j.Logger;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: Modular Robot Project</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 *
 * @author Kvin Drapel
 * @version $Id: TimelinesManager.java,v 1.6 2005/01/27 16:32:34 jaquier Exp $
 */

public class TimelinesManager {

    static Logger logger = Logger.getLogger(TimelinesManager.class);

    static private TimelinesManager m_instance = null;
    private Player m_player = null;
    private double m_endtime = 500.0;
    private double m_increment = 0.2;
    private long m_sleepTime = 500;
    private double m_cTime = 0.0;
    private boolean m_repeat = false;
    private Preferences m_prefs = null;

    /**
     * Private Constructor.
     */
    private TimelinesManager() {
        m_prefs = Preferences.userNodeForPackage(Bluemove.class);
    }

    /**
     * Returns an instance for the singleton.
     * @return TimelinesManager
     */
    static public TimelinesManager getInstance() {
        if (m_instance == null) {
            m_instance = new TimelinesManager();
        }
        return m_instance;
    }

    public boolean getRepeat() {
        return m_repeat;
    }

    public void setRepeat(boolean value) {
        m_repeat = value;
    }

    public double getCurrentTime() {
        return m_cTime;
    }

    public void setCurrentTime(double value) {
        m_cTime = value;
    }

    public Player getPlayer() {
        return m_player;
    }

    /**
     * Creates a new player thread.
     */
    public void play() throws Exception {
        setIncrement(m_prefs.getDouble("playerIncrement", 0.2));
        setSleepTime(m_prefs.getLong("playerSleep", 500));
        setEndTime(this.getLastFrame());
        m_player = new Player(this);
        m_player.setCurrentTime(this.getCurrentTime());
        m_player.setPlayingState(true);
        m_player.start();
        logger.info("Start playing...");
    }

    public void pause() {
        m_player.setPlayingState(false);
        logger.info("Pause playing...");
    }

    /**
     * Stops the thread.
     */
    public void stop() {
        m_player.setPlayingState(false);
        try {
            m_player.join();
        } catch (InterruptedException e) {
            logger.error("Unable to join Player.");
        }
        this.setCurrentTime(0.0);
        logger.info("Stop playing...");
    }

    /**
     * Gets end time.
     * @return double
     */
    public double getEndTime() {
        return m_endtime;
    }

    /**
     * Sets end time.
     * @param value double
     */
    public void setEndTime(double value) {
        m_endtime = value;
    }

    /**
     * Gets time increment.
     * @return double
     */
    public double getIncrement() {
        return m_increment;
    }

    /**
     * Set time increment.
     * @param value double
     */
    public void setIncrement(double value) {
        m_increment = value;
    }

    /**
     * Gets sleep time.
     * @return long
     */
    public long getSleepTime() {
        return m_sleepTime;
    }

    /**
     * Sets sleep time.
     * @param value long
     */
    public void setSleepTime(long value) {
        m_sleepTime = value;
    }

    public double getLastFrame() {
        ModulesManager mm = ModulesManager.getInstance();
        double maxFrame = 0.0;
        for (Module element : mm.getModuleList()) {
            for (ModuleComponent comp : element.getComponentList()) {
                Key key = (Key) comp.getCurve().getKeys().lastElement();
                if (key.getFrame() > maxFrame) {
                    maxFrame = key.getFrame();
                }
            }
        }
        return maxFrame;
    }
}
