package birg.bluemove.timelines.core.timingblock;


/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: Modular Robot Project</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 *
 * @author Kevin Drapel
 * @version $Id: TimingBlock.java,v 1.4 2005/01/26 22:41:43 drapel Exp $
 */
public abstract class TimingBlock implements Comparable {

    protected double m_startTime = -1.0;
    protected double m_endTime = -1.0;
    protected int m_nbRepetitions = -1;

    /**
     * Constructor.
     */
    public TimingBlock() {
    }

    /**
     * Resets the block.
     */
    public abstract void reset();

    /**
     * Returns next time value.
     * @param t double
     * @param delta double
     * @return double
     */
    public abstract double getNextTime(double t, double delta);

    /**
     * Sets range.
     * @param start double
     * @param end double
     */
    public void setRange(double start, double end) {
        setStartTime(start);
        setEndTime(end);
    }

    /**
     * Gets length.
     * @return double
     */
    public double getLength() {
        return m_endTime - m_startTime;
    }

    /**
     * Checks if a time value is inside the block.
     * @param time double
     * @return boolean
     */
    public boolean isInside(double time) {
        if (m_startTime <= time && time < m_endTime) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Sets start time.
     * @param start double
     */
    public void setStartTime(double start) {
        m_startTime = start;
    }

    /**
     * Sets end time.
     * @param end double
     */
    public void setEndTime(double end) {
        m_endTime = end;
    }

    /**
     * Get start time.
     * @return double
     */
    public double getStartTime() {
        return m_startTime;
    }

    /**
     * Gets end time.
     * @return double
     */
    public double getEndTime() {
        return m_endTime;
    }

    /**
     * Sets number of repetitions for this block.
     * @param r int
     */
    public void setNumberRepetitions(int r) {
        m_nbRepetitions = r;
    }

    /**
     * Compares this block with another one according to a precedence criteria.
     * @param o Object
     * @return int
     */
    public int compareTo(java.lang.Object o) {
        if (!(o instanceof TimingBlock)) {
            throw new ClassCastException("A TimingBlock object is expected.");
        }

        TimingBlock anotherBlock = (TimingBlock) o;

        if (getStartTime() < anotherBlock.getStartTime()) {
            return -1;
        }
        if (Math.abs(getStartTime() - anotherBlock.getStartTime()) <
            Double.MIN_VALUE) {
            return 0;
        }
        return 1;
    }

}
