package birg.bluemove.timelines.core.timingblock;

import java.util.Collections;
import java.util.Vector;

import birg.bluemove.timelines.core.interpolator.Key;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: Modular Robot Project</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 *
 * @author Kevin Drapel
 * @version 1.0
 */
public class TimingBlocksManager {

    Vector<TimingBlock> m_timingBlocks = new Vector<TimingBlock>();
    Vector<Key> m_timeKeys = new Vector<Key>();

    double m_totalLength = 0.0;
    double m_playedSequenceStart = 0.0;
    double m_playedSequenceEnd = 0.0;

    /**
     * Constructor.
     */
    public TimingBlocksManager() {
    }

    /**
     * Sets total sequence length.
     * @param length double
     */
    public void setLength(double length) {
        assert (length > 0.0);
        m_totalLength = length;
    }

    /**
     * Sets played sequence range.
     * @param start double
     * @param end double
     */
    public void setPlayedSequence(double start, double end) {
        m_playedSequenceStart = start;
        m_playedSequenceEnd = end;
    }

    /**
     * Checks if two blocks overlap.
     * @param block TimingBlock
     * @return boolean
     */
    public boolean checkIfOverlap(TimingBlock block) {
        for (TimingBlock b : m_timingBlocks) {
            if (block.getStartTime() >= b.getStartTime() &&
                block.getStartTime() < b.getEndTime()) {
                return true;
            }

            if (block.getEndTime() >= b.getStartTime() &&
                block.getEndTime() < b.getEndTime()) {
                return true;
            }
        }

        return false;
    }

    /**
     * Adds a block and checks if it's not overlapping.
     * @param block TimingBlock
     */
    public void addBlock(TimingBlock block) {
        // check that block doesn't overlap on another block
        assert (checkIfOverlap(block) != true):"Block is overlapping";
        m_timingBlocks.add(block);
        Collections.sort(m_timingBlocks);
    }

    /**
     * Remove a block.
     * @param block TimingBlock
     */
    public void removeBlock(TimingBlock block) {
        m_timingBlocks.remove(block);
    }


}
