package birg.bluemove.timelines.gui;

import java.awt.Font;

import javax.swing.JComponent;

import birg.bluemove.timelines.gui.timescale.TimeScale;
import birg.bluemove.utils.gui.FontTools;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: Modular Robot Project</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 *
 * @author Kevin Drapel
 * @version $Id: JTimelineComponent.java,v 1.5 2005/01/26 22:41:43 drapel Exp $
 */
abstract public class JTimelineComponent extends JComponent {

    private boolean m_isExpanded = false;

    protected TimeScale m_timeScale = null;

    private JTimelineComponent m_oppositeComponent = null;

    /**
     * Constructor.
     */
    public JTimelineComponent() {
        super();
    }

    /**
     * Gets expanded height.
     * @return int
     */
    abstract public int getExpandedHeight();

    /**
     * Sets reference to Timescale.
     * @param ts TimeScale
     */
    public void setTimeScale(TimeScale ts) {
        m_timeScale = ts;
    }

    /**
     * Gets reference to Timescale.
     * @return TimeScale
     */
    public TimeScale getTimeScale() {
        return m_timeScale;
    }

    /**
     * Sets opposite component.
     * @param opposite JTimelineComponent
     */
    public void setOppositeComponent(JTimelineComponent opposite) {
        m_oppositeComponent = opposite;
    }

    /**
     * Sets expanded state.
     * @param isExpanded boolean
     */
    public void setExpanded(boolean isExpanded) {
        m_isExpanded = isExpanded;
    }

    /**
     * Answers whetever the component is expanded.
     * @return boolean
     */
    public boolean isExpanded() {
        return m_isExpanded;
    }

    /**
     * Update expanded state.
     */
    final public void updateExpandedState() {
        if (isExpanded()) {
            setSize(this.getWidth(), getExpandedHeight());
        } else {
            setSize(this.getWidth(), getClosedHeight());
        }
    }

    /**
     * Gets closed height.
     * @return int
     */
    final public int getClosedHeight() {
        Font font = FontTools.getControlFont();
        int height = FontTools.getMaxHeight(font);
        return height * 3;
    }

    /**
     * Synchronizes the opposite component.
     */
    protected void synchronizeOpposite() {
        synchronizeOppositeState();
        synchronizeOppositeSize();
    }

    /**
     * Synchronizes the opposite component size.
     */
    protected void synchronizeOppositeSize() {
        m_oppositeComponent.setSize(m_oppositeComponent.getWidth(), getHeight());
    }

    /**
     * Synchronizes the opposite component state.
     */
    protected void synchronizeOppositeState() {
        m_oppositeComponent.setExpanded(isExpanded());
    }

    /**
     * Gets opposite components.
     * @return JComponent
     */
    protected JComponent getOpposite() {
        return m_oppositeComponent;
    }

}
