/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * PlayerLine.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: PlayerLine.java,v 1.1 2005/01/27 16:33:37 jaquier Exp $
 */

package birg.bluemove.timelines.gui;

import java.awt.Color;
import java.awt.Graphics;

import javax.swing.JComponent;

import birg.bluemove.timelines.core.timeline.Player;
import birg.bluemove.timelines.core.timeline.TimelinesManager;
import birg.bluemove.timelines.gui.timescale.TimeScale;
import birg.bluemove.utils.gui.SwingWorker;

public class PlayerLine extends JComponent {

    protected Player m_player = null;
    protected TimelinesManager m_tlManager = TimelinesManager.getInstance();
    protected TimeScale m_timeScale = null;
    protected double m_xpos = 0.0;
    protected SwingWorker m_worker = null;
    private PlayerLine m_playerLine = null;
    private boolean m_isPlaying = false;

    public PlayerLine(TimeScale timeScale) {
        m_timeScale = timeScale;
        m_playerLine = this;
    }

    public void paint(Graphics g) {
        if (m_isPlaying) {
            g.setColor(Color.GRAY);
            g.drawLine((int) m_xpos, 0, (int) m_xpos, this.getHeight());
        }
    }

    public void run() {
        m_worker = new SwingWorker() {
            public Object construct() {
                m_player = m_tlManager.getPlayer();
                double xpos = 0.0;

                m_timeScale.setStartFrame(m_player.getCurrentTime());
                m_timeScale.repaint();
                while (m_player.getPlayingState()) {
                    double cTime = m_player.getCurrentTime();
                    m_xpos = m_timeScale.getPixelFromFrame(cTime);

                    if (m_xpos > m_timeScale.getWidth() / 1.5) {
                        m_timeScale.setStartFrame(cTime);
                        m_timeScale.repaint();
                    } else if (xpos > m_xpos) {
                        m_timeScale.setStartFrame(m_player.getCurrentTime());
                        m_timeScale.repaint();
                    }
                    xpos = m_xpos;

                    m_playerLine.repaint();
                    try {
                        Thread.sleep(80);
                    } catch (InterruptedException e) {
                    }
                }
                return null;
            }

            public void finished() {
                m_isPlaying = false;
                m_playerLine.repaint();
            }
        };
        m_isPlaying = true;
        m_worker.start();
    }
}
