/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * TimelinesTab.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: TimelinesTab.java,v 1.25 2005/02/09 15:18:03 jaquier Exp $
 */

package birg.bluemove.timelines.gui;

import java.awt.Point;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;

import birg.bluemove.module.core.Module;
import birg.bluemove.module.core.ModulesManager;
import birg.bluemove.timelines.core.timeline.TimelinesManager;
import birg.bluemove.timelines.gui.infotree.ModuleInfoNode;
import birg.bluemove.timelines.gui.infotree.ModulesTree;
import birg.bluemove.timelines.gui.infotree.nodes.ModuleComponentInfoNode;
import birg.bluemove.timelines.gui.timelinestree.ModuleTimelineNode;
import birg.bluemove.timelines.gui.timelinestree.TimelinesTree;
import birg.bluemove.timelines.gui.timelinestree.nodes.
        ModuleComponentTimelineNode;
import birg.bluemove.timelines.gui.timescale.TimeScale;
import birg.bluemove.utils.gui.GUITools;
import birg.bluemove.utils.gui.MessageBox;
import org.apache.log4j.Logger;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: Modular Robot Project</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 *
 * @author Kevin Drapel
 * @version 1.0
 */
public class TimelinesTab extends JPanel implements ComponentListener, Observer,
        ActionListener {

    static Logger logger = Logger.getLogger(TimelinesTab.class);
    ModulesTree m_modulesTree = null;
    TimelinesTree m_timelinesTree = null;
    TimeScale m_timeScale = null;
    JScrollBar m_vScroll = null;
    protected boolean m_isPlaying = false;

    // player / timelscale control

    JButton m_play = null;
    JButton m_stop = null;
    JButton m_zoomIn = null;
    JButton m_zoomOut = null;
    JToggleButton m_repeat = null;
    PlayerLine m_line = null;

    ModulesManager m_mm = ModulesManager.getInstance();
    TimelinesManager m_tlM = TimelinesManager.getInstance();

    public enum ToolType {
        ADD_LINEAR, ADD_SPLINE
    }


    static public ToolType m_currentToolType = ToolType.ADD_SPLINE;


    /**
     * Constructor.
     */
    public TimelinesTab() {
        try {
            initialize();

            /*Module m1 = new Module("module 1", "0x000001");
                         Module m2 = new Module("module 2", "0x000002");
                         Module m3 = new Module("module 3", "0x000003");

                         m_mm.addModule(m1);
                         m_mm.addModule(m2);
                         m_mm.addModule(m3);

                         Actuator a1 = new Actuator(m1, "actuator 1");
                         m1.addComponent(a1);
                         a1.setRange( -20.0, 60.0);
                         Actuator a2 = new Actuator(m1, "actuator 2");
                         a2.setRange( -100.0, 100.0);
                         m1.addComponent(a2);

                         m2.addComponent(new Actuator(m1, "actuator 1"));
                         m2.addComponent(new Actuator(m2, "actuator 2"));
                         m3.addComponent(new Sensor(m1, "sensor 1"));

                         m3.addComponent(new Actuator(m1, "actuator 1"));
                         m3.addComponent(new Actuator(m2, "actuator 2"));
                         m3.addComponent(new Actuator(m2, "actuator 3"));*/

        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    /**
     * Initializes the main timelines tab.
     * @throws Exception
     */
    private void initialize() throws Exception {
        this.setLayout(null);
        this.addComponentListener(this);

        m_mm.addObserver(this);

        m_modulesTree = new ModulesTree(m_mm, this);
        m_timelinesTree = new TimelinesTree(m_mm, this);
        m_timeScale = new TimeScale(m_modulesTree, m_timelinesTree);
        m_line = new PlayerLine(m_timeScale);

        m_modulesTree.setLocation(20, 20);
        m_modulesTree.setSize(300, 450);

        m_timelinesTree.setLocation(300 + 20, 20);
        m_timelinesTree.setSize(500, 450);

        m_timeScale.setLocation(m_timelinesTree.getX(),
                                m_timelinesTree.getY() +
                                m_timelinesTree.getHeight());

        this.add(m_line);
        this.add(m_modulesTree);
        this.add(m_timelinesTree);
        this.add(m_timeScale);

        linkComponents(m_modulesTree, m_timelinesTree, m_timeScale);

        m_vScroll = new JScrollBar();
        this.add(m_vScroll);

        m_vScroll.addAdjustmentListener(new
                                        jScrollBar1_adjustmentAdapter(this));

        // player / timescale controls
        m_play = new JButton(GUITools.getImageIcon("control_play.png"));
        m_stop = new JButton(GUITools.getImageIcon("control_stop.png"));
        m_zoomIn = new JButton(GUITools.getImageIcon("control_zoomin.png"));
        m_zoomOut = new JButton(GUITools.getImageIcon(
                "control_zoomout.png"));
        m_repeat = new JToggleButton(GUITools.getImageIcon(
                "control_repeat.png"));

        m_play.setSize(26, 26);
        m_stop.setSize(26, 26);
        m_zoomIn.setSize(26, 26);
        m_zoomOut.setSize(26, 26);
        m_repeat.setSize(26, 26);
        m_stop.setEnabled(false);

        this.add(m_play);
        this.add(m_stop);
        this.add(m_zoomIn);
        this.add(m_zoomOut);
        this.add(m_repeat);

        m_play.addActionListener(this);
        m_stop.addActionListener(this);
        m_zoomIn.addActionListener(this);
        m_zoomOut.addActionListener(this);
        m_repeat.addActionListener(this);
    }


    /**
     * Catches the events fired by the player / zoom.
     * @param e ActionEvent
     */
    public void actionPerformed(ActionEvent e) {
        Object o = e.getSource();
        if (o == m_play) {
            if (!m_isPlaying) {
                startPlaying();
            } else {
                pausePlaying();
            }
        }

        if (o == m_stop) {
            stopPlaying();
        }

        if (o == m_zoomIn) {
            m_timeScale.increaseZoom();
            m_timeScale.repaint();
            m_timelinesTree.repaint();
        }

        if (o == m_zoomOut) {
            m_timeScale.decreaseZoom();
            m_timeScale.repaint();
            m_timelinesTree.repaint();
        }

        if (o == m_repeat) {
            if (m_repeat.isSelected()) {
                m_tlM.setRepeat(true);
            } else {
                m_tlM.setRepeat(false);
            }
        }
    }

    /**
     * Starts playing.
     */
    public void startPlaying() {
        try {
            m_tlM.play();
            m_isPlaying = true;
            m_play.setIcon(GUITools.getImageIcon("control_pause.png"));
            m_stop.setEnabled(true);
            m_line.run();
        } catch (Exception e) {
            logger.error(e.getMessage());
            String msg = "Unable to play. A communication system was not found";
            MessageBox mBox = new MessageBox(null,
                                             MessageBox.MessageBoxType.ERROR,
                                             msg);
            mBox.showDialog();
        }
    }

    public void pausePlaying() {
        m_isPlaying = false;
        m_play.setIcon(GUITools.getImageIcon("control_play.png"));
        m_stop.setEnabled(true);
        m_tlM.pause();
    }

    /**
     * Stops playing.
     */
    public void stopPlaying() {
        m_isPlaying = false;
        m_play.setIcon(GUITools.getImageIcon("control_play.png"));
        m_stop.setEnabled(false);
        m_tlM.stop();
    }

    /**
     * Links components together.
     * @param modules ModulesTree
     * @param timelines TimelinesTree
     * @param timeScale TimeScale
     */
    private void linkComponents(ModulesTree modules,
                                TimelinesTree timelines, TimeScale timeScale) {

        Vector<ModuleInfoNode> modulesNodes = modules.getNodes();
        Vector<ModuleTimelineNode> timelinesNodes = timelines.getNodes();

        assert (modulesNodes.size() == timelinesNodes.size());

        modules.setTimelinesTree(timelines);
        timelines.setModulesTree(modules);

        for (int n = 0; n < modulesNodes.size(); n++) {

            modulesNodes.get(n).setOppositeComponent(timelinesNodes.get(n));
            timelinesNodes.get(n).setOppositeComponent(modulesNodes.get(n));

            Vector<ModuleComponentInfoNode>
                    componentsLeft = modulesNodes.get(n).getComponentInfoNodes();

            Vector<ModuleComponentTimelineNode>
                    componentsRight = timelinesNodes.get(n).
                                      getComponentTimelineNodes();

            for (int i = 0; i < componentsLeft.size(); i++) {
                // create a navigable link between left and right components
                componentsLeft.get(i).setOppositeComponent(componentsRight.get(
                        i));
                componentsRight.get(i).setOppositeComponent(componentsLeft.get(
                        i));

                // create a navigable link between components and scale
                componentsLeft.get(i).setTimeScale(m_timeScale);
                componentsRight.get(i).setTimeScale(m_timeScale);
            }
        }

    }

    /**
     * Fired when tab is resized.
     * @param e ComponentEvent
     */
    public void componentResized(ComponentEvent e) {
        int height = getHeight();
        // Resize the module tree
        m_modulesTree.setSize(m_modulesTree.getWidth(), height - 60);
        m_modulesTree.resetSizeLocation();

        // Resize the timelines
        m_timelinesTree.setSize(getWidth() - 350, height - 60);
        m_timelinesTree.resetSizeLocation();

        m_timeScale.setLocation(m_timelinesTree.getX(),
                                m_timelinesTree.getY() +
                                m_timelinesTree.getHeight());
        m_timeScale.setSize(m_timelinesTree.getWidth(), 35);

        // Resize player line
        m_line.setLocation(m_timelinesTree.getX(), m_timelinesTree.getY());
        m_line.setSize(m_timelinesTree.getWidth(), m_timelinesTree.getHeight());

        setScrollbarDimension();

        int hTime = m_timeScale.getHeight();
        int xPlayer = 20;
        int yPlayer = m_timeScale.getY() + (hTime - 26) / 2;
        m_play.setLocation(xPlayer + 30 * 0, yPlayer);
        m_stop.setLocation(xPlayer + 30 * 1, yPlayer);
        m_repeat.setLocation(xPlayer + 30 * 3, yPlayer);

        xPlayer = m_timeScale.getX();
        m_zoomIn.setLocation(xPlayer - 30 * 1, yPlayer);
        m_zoomOut.setLocation(xPlayer - 30 * 2, yPlayer);
    }

    public void componentMoved(ComponentEvent e) {
    }

    /**
     * Fired when tab is shown.
     * @param e ComponentEvent
     */
    public void componentShown(ComponentEvent e) {
        logger.warn("linkModulesToTimelines: bad hack, should be fixed !");
        linkComponents(m_modulesTree, m_timelinesTree, m_timeScale);
        m_modulesTree.refreshModulesLocation();
        m_timelinesTree.refreshModulesLocation();

        setScrollbarDimension();
    }

    /**
     * Sets scrollbar dimension.
     */
    public void setScrollbarDimension() {
        Point p = m_timelinesTree.getLocation();
        m_vScroll.setSize(20, m_timelinesTree.getHeight());
        m_vScroll.setLocation(p.x + m_timelinesTree.getWidth() + 4, p.y);
    }

    /**
     * Sets scrollbar range.
     */
    public void setScrollbarRange() {
        m_vScroll.setMinimum(0);
        int totalHeight = m_modulesTree.getCurrentHeight();
        int windowHeight = m_modulesTree.getHeight();
        if (totalHeight < windowHeight) {
            m_vScroll.setEnabled(false);
        } else {
            m_vScroll.setEnabled(true);
            m_vScroll.setMaximum(m_modulesTree.getCurrentHeight() -
                                 windowHeight);
        }
    }

    public void componentHidden(ComponentEvent e) {
    }

    /**
     * Updates when the modules manager has changed.
     * @param o Observable
     * @param arg Object
     */
    public void update(Observable o, Object arg) {
        logger.debug("Update observer --> " + this.getClass().getName());
        if (o instanceof ModulesManager) {
            if (arg.toString().equals("add")) {
                Module m = m_mm.getLastModule();
                m_modulesTree.addModule(m);
                m_timelinesTree.addModule(m);

                linkComponents(m_modulesTree, m_timelinesTree, m_timeScale);
                m_modulesTree.resetSizeAll();
                /*
                                m_modulesTree.resetSizeLocation();
                                m_timelinesTree.resetSizeLocation();

                                m_modulesTree.refreshModulesLocation();
                                m_timelinesTree.refreshModulesLocation();
                 */

                m_timelinesTree.repaintComponents();
                m_modulesTree.repaintComponents();

            } else if (arg.toString().equals("remove")) {
                Module m = m_mm.getLastRemovedModule();
                m_modulesTree.removeModule(m);
                m_timelinesTree.removeModule(m);
                linkComponents(m_modulesTree, m_timelinesTree, m_timeScale);
                m_modulesTree.resetSizeLocation();
                m_modulesTree.refreshModulesLocation();

                m_timelinesTree.repaintComponents();
                m_modulesTree.repaintComponents();
            } else if (arg.toString().equals("clear")) {
                m_modulesTree.clear();
                m_timelinesTree.clear();
                linkComponents(m_modulesTree, m_timelinesTree, m_timeScale);
                this.repaint();
            }
        }
    }

    /**
     * Updates if scrollbar has moved.
     * @param e AdjustmentEvent
     */
    public void jScrollBar1_adjustmentValueChanged(AdjustmentEvent e) {
        m_modulesTree.setVerticalOffset( -e.getValue());
        m_timelinesTree.setVerticalOffset( -e.getValue());

        m_modulesTree.refreshModulesLocation();
        m_timelinesTree.refreshModulesLocation();
        m_modulesTree.repaintComponents();
        m_timelinesTree.repaintComponents();
    }

}


class jScrollBar1_adjustmentAdapter implements AdjustmentListener {
    private TimelinesTab adaptee;
    jScrollBar1_adjustmentAdapter(TimelinesTab adaptee) {
        this.adaptee = adaptee;
    }

    public void adjustmentValueChanged(AdjustmentEvent e) {
        adaptee.jScrollBar1_adjustmentValueChanged(e);
    }
}
