/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * ModuleComponentInfoNode.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: ModuleComponentInfoNode.java,v 1.3 2005/01/26 22:41:43 drapel Exp $
 */


package birg.bluemove.timelines.gui.infotree.nodes;

import java.awt.Font;
import java.awt.Graphics;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import birg.bluemove.module.core.ModuleComponent;
import birg.bluemove.timelines.gui.JTimelineComponent;
import birg.bluemove.utils.gui.FontTools;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: Modular Robot Project</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 *
 * @author Kevin Drapel
 * @version 1.0
 */
public abstract class ModuleComponentInfoNode extends JTimelineComponent implements
        MouseListener, MouseMotionListener {

    protected ModuleComponent m_moduleComponent = null;
    protected int m_currentHeight = 0;
    boolean m_customHeightEnabled = false;
    int m_customHeight = 0;

    /**
     * Constructor.
     * @param moduleComponent ModuleComponent
     */
    public ModuleComponentInfoNode(ModuleComponent moduleComponent) {
        this.setLayout(null);
        m_moduleComponent = moduleComponent;

        this.addMouseListener(this);
        this.addMouseMotionListener(this);
    }

    /**
     * Gets the component linked to this node.
     * @return ModuleComponent
     */
    public ModuleComponent getComponent() {
        return m_moduleComponent;
    }


    /**
     * Paints the border of the component.
     * @param g Graphics
     */
    public abstract void paintBorder(Graphics g);

    /**
     * Paints the component.
     * @param g Graphics
     */
    public abstract void paintComponent(Graphics g);


    /**
     * Sets current dimension of the component.
     * @param currentWidth int
     * @param currentHeight int
     */
    public void setCurrentDimension(int currentWidth, int currentHeight) {
        this.setSize(currentWidth, currentHeight);
        m_currentHeight = currentHeight;
    }

    /**
     * Gets height of component when it is expanded.
     * @return int
     */
    public int getExpandedHeight() {
        if (!m_customHeightEnabled) {
            Font font = FontTools.getControlFont();
            int height = FontTools.getMaxHeight(font);
            return height * 10;
        } else {
            return m_customHeight;
        }
    }


}
