package birg.bluemove.timelines.gui.infotree.nodes;


import java.awt.*;
import java.awt.event.*;
import java.util.HashMap;

import javax.swing.ImageIcon;

import birg.bluemove.module.core.Actuator;
import birg.bluemove.module.core.ModuleComponent;
import birg.bluemove.resources.resources;
import birg.bluemove.utils.gui.FontTools;
import birg.bluemove.utils.gui.GUITools;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: Modular Robot Project</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 *
 * @author not attributable
 * @version 1.0
 */
public class SensorInfoNode extends ModuleComponentInfoNode implements
        MouseListener, MouseMotionListener {

    HashMap<String, Rectangle> m_activationRect;

    Color m_colorIsActive = new Color(0.90f, 1.0f, 0.9f);
    Color m_colorIsInactive = new Color(1.0f, 0.90f, 0.6f);

    public SensorInfoNode(ModuleComponent moduleComponent) {
        super(moduleComponent);
        // hashmap for activation regions (icons)
        m_activationRect = new HashMap<String, Rectangle>();
    }

    /**
     * Paints border.
     * @param g Graphics
     */
    public void paintBorder(Graphics g) {
        //super.paintBorder(g);
        Graphics2D g2d = (Graphics2D) g.create();
        Rectangle bounds = new Rectangle(0, 0, getWidth(), getHeight());
        g2d.setColor(Color.BLACK);
        g2d.drawRect(bounds.x, bounds.y, bounds.width - 1, bounds.height);
        g2d.dispose();
    }


    /**
     * Paints component.
     * @param g Graphics
     */
    public void paintComponent(Graphics g) {
        //  super.paintComponent(g);
        Graphics2D g2d = (Graphics2D) g.create();
        g2d.setRenderingHints(GUITools.getBestQualityHints());

        Rectangle bounds = new Rectangle(0, 0, getWidth(), getHeight());
        GradientPaint gradient;

        gradient = new GradientPaint(0, 0, m_colorIsInactive,
                                     0, this.getClosedHeight() * 2,
                                     Color.WHITE,
                                     false);

        g2d.setPaint(gradient);
        g2d.fillRect(bounds.x, bounds.y, bounds.width, bounds.height);

        // draw icons and store the activation rectangles in a hashmap
        int yIcon = (getClosedHeight() - 16) / 2;
        Rectangle r1 = GUITools.drawIcon(g2d, getIconLed(), 5 + 16, yIcon);
        Rectangle r2 = GUITools.drawIcon(g2d, getIconArrow(), 5 + 5 + 16 + 16,
                                         yIcon);

        m_activationRect.put("Icon.Enabled", r1);
        m_activationRect.put("Icon.Expanded", r2);

        Font font = FontTools.getControlFont();
        int fontHeight = FontTools.getMaxHeight(font);

        int yString = fontHeight + (getClosedHeight() - fontHeight) / 2;

        g2d.setColor(Color.BLACK);
        g2d.drawString(m_moduleComponent.getName(), 5 + 5 + 16 + 16 + 16 + 10,
                       yString);

        g2d.setFont(FontTools.getControlFontItalic());

        GUITools.drawIcon(g2d, getIconComponent(), 5 + 5 + 16 + 16 + 16 + 100,
                          yIcon);

        g2d.drawString(getDescriptionComponent(),
                       5 + 5 + 16 + 16 + 16 + 116 + 10, yString);

        g2d.dispose();
    }

    /**
     * Gets icon for the component.
     * @return ImageIcon
     */
    protected ImageIcon getIconComponent() {
        if (m_moduleComponent instanceof Actuator) {
            return GUITools.getImageIcon("icon_actuator.png");
        } else {
            return GUITools.getImageIcon("icon_sensor.png");
        }
    }

    /**
     * Creates a summary of the sensor description.
     * @return String
     */
    protected String getDescriptionComponent() {
        if (m_moduleComponent instanceof Actuator) {
            Actuator actuator = (Actuator) m_moduleComponent;
            double min = actuator.getRangeMin();
            double max = actuator.getRangeMax();
            return "[" + Double.toString(min) + ";" + Double.toString(max) +
                    "]";
        } else {
            return "sensor";
        }
    }


    /**
     * Gets LED icon.
     * @return ImageIcon
     */
    protected ImageIcon getIconLed() {
        return new ImageIcon(resources.class.getResource(
                "icon_redlight.png"));
    }

    /**
     * Gets arrow icon.
     * @return ImageIcon
     */
    protected ImageIcon getIconArrow() {
        return new ImageIcon(resources.class.getResource(
                "icon_arrowright.png"));
    }

    public void mouseDragged(MouseEvent e) {
    }

    public void mouseMoved(MouseEvent e) {
    }

    public void mouseClicked(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

}
