/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * V2D.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: V2D.java,v 1.3 2005/01/26 22:41:43 drapel Exp $
 */

package birg.bluemove.utils;

import java.awt.geom.Point2D;


public class V2D extends Point2D.Double implements Cloneable {

    /** Creates a new instance of v2d */
    public V2D() {
    }

    /**
     * Creates a new instance of v2d.
     * @param vx double
     * @param vy double
     */
    public V2D(double vx, double vy) {
        super(vx, vy);
    }

    /**
     * Add two vectors.
     * @param a V2D
     * @param b V2D
     * @return V2D
     */
    public static V2D add(V2D a, V2D b) {
        V2D result = new V2D();
        result.x = a.x + b.x;
        result.y = a.y + b.y;
        return result;
    }

    /**
     * Subtracts two vectors.
     * @param a V2D
     * @param b V2D
     * @return V2D
     */
    public static V2D sub(V2D a, V2D b) {
        V2D result = new V2D();
        result.x = a.x - b.x;
        result.y = a.y - b.y;
        return result;
    }

    /**
     * Multiplies a vector by a scalar value.
     * @param a V2D
     * @param factor double
     * @return V2D
     */
    public static V2D mul(V2D a, double factor) {
        V2D result = new V2D();
        result.x = a.x * factor;
        result.y = a.y * factor;
        return result;
    }

    /**
     * Divides a vector by a scalar value.
     * @param a V2D
     * @param factor double
     * @return V2D
     */
    public static V2D div(V2D a, double factor) {
        V2D result = new V2D();
        result.x = a.x / factor;
        result.y = a.y / factor;
        return result;
    }

    /**
     * Normalizes the vector.
     * @param a V2D
     * @return V2D
     */
    public static V2D normalize(V2D a) {
        V2D result = new V2D();
        double l = length(a);
        if (l == 0.0) {
            return new V2D(0.0, 0.0);
        }
        result.x = a.x / l;
        result.y = a.y / l;
        return result;
    }

    /**
     * Computes the dot product between two vectors.
     * @param a V2D
     * @param b V2D
     * @return double
     */
    public static double dotProduct(V2D a, V2D b) {
        return a.x * b.x + a.y * b.y;
    }

    /**
     * Returns the length of the vector
     * @param a V2D
     * @return double
     */
    public static double length(V2D a) {
        return Math.sqrt(a.x * a.x + a.y * a.y);
    }

}
