/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * About.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: About.java,v 1.3 2005/01/26 22:41:43 drapel Exp $
 */

package birg.bluemove.utils.gui;

import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.*;
import javax.swing.border.TitledBorder;

import birg.bluemove.resources.resources;

/**
 *
 * <p>Title: Bluemove</p>
 *
 * <p>Description: Modular Robot Project</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 *
 * @author Cyril Jaquier
 * @version 1.0
 */
public class About extends JDialog {

    ImageIcon m_img = null;

    /**
     * Constructor
     */
    public About() {
        m_img = new ImageIcon(resources.class.getResource("splashscreen.png"));

        try {
            jbInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }

        String field[] = {"Name", "Value"};
        String table[][] = { {"Java VM vendor",
                           System.getProperty("java.vm.vendor")},
                           {"Java VM version",
                           System.getProperty("java.vm.version")}, {"Java home",
                           System.getProperty("java.home")}, {"OS arch",
                           System.getProperty("os.arch")}, {"OS name",
                           System.getProperty("os.name")}, {"OS version",
                           System.getProperty("os.version")}, {"File encoding",
                           System.getProperty("file.encoding")}
        };
        jPanel1.remove(jTable1);
        jTable1 = new JTable(table, field);
        jPanel1.add(jTable1, java.awt.BorderLayout.CENTER);
        this.pack();
    }

    /**
     * Initialize the about box.
     * @throws Exception
     */
    private void jbInit() throws Exception {
        TextScroller txt = new TextScroller();
        txt.setSize(300, 150);
        this.setSize(m_img.getIconWidth() + 300, m_img.getIconHeight());
        jLabel1.setText("");
        jLabel1.setIcon(m_img);
        jPanel1.setLayout(borderLayout2);
        this.addMouseListener(new About_this_mouseAdapter(this));
        jPanel1.setBackground(Color.white);
        jPanel1.setBorder(BorderFactory.createEmptyBorder(0, 6, 0, 6));
        jTable1.setBorder(null);
        jLabel2.setBorder(BorderFactory.createEmptyBorder(0, 0, 10, 0));
        jLabel2.setHorizontalAlignment(SwingConstants.CENTER);
        this.setResizable(false);
        this.setUndecorated(true);
        jPanel2.setLayout(borderLayout3);
        jLabel2.setText("Bluemove");
        jPanel2.setBorder(BorderFactory.createRaisedBevelBorder());
        jPanel1.add(jTable1, java.awt.BorderLayout.CENTER);
        jPanel1.add(txt, java.awt.BorderLayout.SOUTH);
        this.getContentPane().add(jPanel2, java.awt.BorderLayout.CENTER);
        jPanel2.add(jPanel1, java.awt.BorderLayout.EAST);
        jPanel2.add(jLabel1, java.awt.BorderLayout.WEST);
        jPanel1.add(jLabel2, java.awt.BorderLayout.NORTH);
    }

    JLabel jLabel1 = new JLabel();
    JPanel jPanel1 = new JPanel();
    JLabel jLabel2 = new JLabel();
    JTable jTable1 = new JTable();
    BorderLayout borderLayout2 = new BorderLayout();
    TitledBorder titledBorder1 = new TitledBorder("");
    JPanel jPanel2 = new JPanel();
    BorderLayout borderLayout3 = new BorderLayout();

    /**
     * Closes the dialog after a mouse press.
     * @param e MouseEvent
     */
    public void this_mousePressed(MouseEvent e) {
        this.dispose();
    }
}


/**
 *
 * <p>Title: Bluemove</p>
 *
 * <p>Description: Modular Robot Project</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 *
 * @author Cyril Jaquier
 * @version 1.0
 */
class TextScroller extends JComponent {

    int currentY = 150;
    String credits[] = {"BIRG @ EPFL", "", "Supervisor:", "Auke Ijspeert",
                       "", "Assistants:", "Alessandro Crespi", "Andres Upegui",
                       "", "Coders:", "Kvin Drapel", "Cyril Jaquier"};

    /**
     * Constructor.
     */
    public TextScroller() {
    }

    /**
     * Renders a scrolling with credits.
     * @param g Graphics
     */
    public void paintComponent(Graphics g) {
        Graphics2D g2d = (Graphics2D) g;
        g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
                             RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

        setFont(getFont().deriveFont(Font.BOLD));

        int width = getWidth();
        int height = getHeight();
        int strHeight = getFontMetrics(getFont()).getHeight();

        int y = currentY;
        for (int i = 0; i < credits.length; i++) {
            float color = (float) y / (float) height;
            g.setColor(Color.getHSBColor(0.67f, color, 1.0f));

            int strLength = getFontMetrics(getFont()).stringWidth(credits[i]);
            int center = (int) ((float) width / 2.0f - (float) strLength / 2.0f);
            center = center + (int) (30.0f * Math.sin(((float) y) / 20.0f));

            g.drawString(credits[i], center, y);
            y += strHeight;
        }

        currentY--;
        if (currentY < -strHeight * credits.length) {
            currentY = height;
        }

        repaint();
        try {
            Thread.sleep(40);
        } catch (Exception e) {
        }
    }

    /**
     * Returns the preferred size.
     * @return Dimension
     */
    public Dimension getPreferredSize() {
        return new Dimension(300, 150);
    }
}


class About_this_mouseAdapter extends MouseAdapter {
    private About adaptee;
    About_this_mouseAdapter(About adaptee) {
        this.adaptee = adaptee;
    }

    public void mousePressed(MouseEvent e) {
        adaptee.this_mousePressed(e);
    }
}
