/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * MessageBox.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: MessageBox.java,v 1.6 2005/02/09 15:18:03 jaquier Exp $
 */

package birg.bluemove.utils.gui;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;
import javax.swing.border.EmptyBorder;

import birg.bluemove.resources.resources;

/**
 *
 * <p>Title: Bluemove</p>
 *
 * <p>Description: Modular Robot Project</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 *
 * @author Kevin Drapel
 * @version 1.0
 */

public class MessageBox extends JDialog {

    public static enum MessageBoxType {
        WARNING, QUESTION, INFO, ERROR
    }


    public static enum MessageBoxAnswer {
        YES, NO, OK
    }


    protected MessageBoxType m_msgType;
    protected MessageBoxAnswer m_answer;

    /**
     * Constructor.
     * @param owner Frame
     * @param msgType MessageBoxType
     * @param msg String
     */
    public MessageBox(Frame owner, MessageBoxType msgType, String msg) {
        super(owner);
        this.setDefaultCloseOperation(javax.swing.WindowConstants.
                                      DO_NOTHING_ON_CLOSE);
        initComponents();
        init(msgType, msg);
        this.setModal(true);

    }

    /**
     * Initialization.
     * @param msgType MessageBoxType
     * @param msg String
     */
    protected void init(MessageBoxType msgType, String msg) {
        m_buttonLeft.addActionListener(new MessageBox_buttonLeft_actionAdapter(this));
        m_buttonRight.addActionListener(new
                                        MessageBox_buttonRight_actionAdapter(this));
        m_txtArea.setText(msg);

        m_msgType = msgType;
        switch (msgType) {
        case WARNING:
            this.setTitle("Warning");
            m_buttonLeft.setText("OK");
            m_buttonRight.setVisible(false);
            m_labelIcon.setIcon(new ImageIcon(resources.class.getResource(
                    "msgbox_warning.png")));
            break;
        case INFO:
            this.setTitle("Information");
            m_buttonLeft.setText("OK");
            m_buttonRight.setVisible(false);
            m_labelIcon.setIcon(new ImageIcon(resources.class.getResource(
                    "msgbox_info.png")));
            break;
        case QUESTION:
            this.setTitle("Question");
            m_buttonLeft.setText("Yes");
            m_buttonRight.setText("No");
            m_labelIcon.setIcon(new ImageIcon(resources.class.getResource(
                    "msgbox_question.png")));
            break;
        case ERROR:
            this.setTitle("Error");
            m_buttonLeft.setText("OK");
            m_buttonRight.setVisible(false);
            m_labelIcon.setIcon(new ImageIcon(resources.class.getResource(
                    "msgbox_error.png")));
            break;
        }
    }

    /**
     * Displays the dialogbox.
     */
    public void showDialog() {
        int nbRows = m_txtArea.getLineCount();
        // dirty hack...
        this.setSize(new Dimension(350, 130 + nbRows * 15));
        setLocationRelativeTo(getParent());
        setVisible(true);
        toFront();
    }

    /**
     * Gets the answer returned by the dialogbox.
     * @return MessageBoxAnswer
     */
    public MessageBoxAnswer getAnswer() {
        return m_answer;
    }

    /**
     * Initialize the components in the dialogbox.
     */
    private void initComponents() {
        dialogPane = new JPanel();
        contentPane = new JPanel();
        m_txtArea = new JTextArea(80, 20);
        m_labelIcon = new JLabel();
        panel1 = new JPanel();
        m_buttonLeft = new JButton();
        m_buttonRight = new JButton();

        Container contentPane2 = getContentPane();
        contentPane2.setLayout(new BorderLayout());

        dialogPane.setLayout(new BorderLayout());

        contentPane.setLayout(new BorderLayout());

        m_txtArea.setBackground(SystemColor.control);
        m_txtArea.setOpaque(false);
        m_txtArea.setEditable(false);
        m_txtArea.setHighlighter(null);
        m_txtArea.setLineWrap(true);
        m_txtArea.setWrapStyleWord(true);
        contentPane.add(m_txtArea, BorderLayout.CENTER);

        dialogPane.add(contentPane, BorderLayout.CENTER);

        contentPane2.add(dialogPane, BorderLayout.CENTER);

        m_labelIcon.setBorder(null);
        m_labelIcon.setVerticalAlignment(SwingConstants.TOP);
        contentPane2.add(m_labelIcon, BorderLayout.WEST);

        panel1.setLayout(new FlowLayout());

        m_buttonLeft.setText("OK");
        panel1.add(m_buttonLeft);

        m_buttonRight.setText("Cancel");
        panel1.add(m_buttonRight);

        m_labelIcon.setBorder(new EmptyBorder(16, 6, 6, 6));
        m_txtArea.setBorder(new EmptyBorder(16, 6, 6, 6));
        contentPane2.add(panel1, BorderLayout.SOUTH);
    }

    /**
     * Event handler for left button.
     * @param actionEvent ActionEvent
     */
    public void leftButton_actionPerformed(ActionEvent actionEvent) {
        switch (m_msgType) {
        case WARNING:
            m_answer = MessageBoxAnswer.OK;
            break;
        case ERROR:
            m_answer = MessageBoxAnswer.OK;
            break;
        case INFO:
            m_answer = MessageBoxAnswer.OK;
            break;
        case QUESTION:
            m_answer = MessageBoxAnswer.YES;
            break;
        }
        this.dispose();
    }

    /**
     * Event handler for right button.
     * @param actionEvent ActionEvent
     */
    public void rightButton_actionPerformed(ActionEvent actionEvent) {
        switch (m_msgType) {
        case QUESTION:
            m_answer = MessageBoxAnswer.NO;
            break;
        }
        this.dispose();
    }

    private JPanel dialogPane;
    private JPanel contentPane;
    private JTextArea m_txtArea;
    private JLabel m_labelIcon;
    private JPanel panel1;
    private JButton m_buttonLeft;
    private JButton m_buttonRight;
}


class MessageBox_buttonLeft_actionAdapter implements ActionListener {
    private MessageBox adaptee;
    MessageBox_buttonLeft_actionAdapter(MessageBox adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent actionEvent) {
        adaptee.leftButton_actionPerformed(actionEvent);
    }
}


class MessageBox_buttonRight_actionAdapter implements ActionListener {
    private MessageBox adaptee;
    MessageBox_buttonRight_actionAdapter(MessageBox adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent actionEvent) {
        adaptee.rightButton_actionPerformed(actionEvent);
    }
}
